# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch,
    call
)

# Import 3rd-party libs
from salt.ext import six

# Import Salt Libs
import salt.states.proxy as proxy


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ProxyTestCase(TestCase, LoaderModuleMockMixin):
    '''
        Validate the proxy state
    '''
    def setup_loader_modules(self):
        return {proxy: {}}

    def test_set_proxy_macos(self):
        '''
            Test to make sure we can set the proxy settings on macOS
        '''
        with patch.dict(proxy.__grains__, {'os': 'Darwin'}):
            expected = {'changes': {
                'new': [
                    {'port': '3128',
                     'server': '192.168.0.1',
                     'service': 'http',
                     'user': 'frank'},
                    {'port': '3128',
                     'server': '192.168.0.1',
                     'service': 'https',
                     'user': 'frank'},
                    {'port': '3128',
                     'server': '192.168.0.1',
                     'service': 'ftp',
                     'user': 'frank'},
                    {'bypass_domains': ['salt.com', 'test.com']}]
                },
                'comment': 'http proxy settings updated correctly\nhttps proxy settings updated correctly\nftp proxy '
                           'settings updated correctly\nProxy bypass domains updated correctly\n',
                'name': '192.168.0.1',
                'result': True}

            set_proxy_mock = MagicMock(return_value=True)
            patches = {
                'proxy.get_http_proxy': MagicMock(return_value={}),
                'proxy.get_https_proxy': MagicMock(return_value={}),
                'proxy.get_ftp_proxy': MagicMock(return_value={}),
                'proxy.get_proxy_bypass': MagicMock(return_value=[]),
                'proxy.set_http_proxy': set_proxy_mock,
                'proxy.set_https_proxy': set_proxy_mock,
                'proxy.set_ftp_proxy': set_proxy_mock,
                'proxy.set_proxy_bypass': set_proxy_mock,
            }

            with patch.dict(proxy.__salt__, patches):
                out = proxy.managed('192.168.0.1', '3128', user='frank', password='passw0rd',
                                    bypass_domains=['salt.com', 'test.com'])
                if six.PY3:
                    # Sorting is different in Py3
                    out['changes']['new'][-1]['bypass_domains'] = sorted(out['changes']['new'][-1]['bypass_domains'])

                calls = [
                    call('192.168.0.1', '3128', 'frank', 'passw0rd', 'Ethernet'),
                    call('192.168.0.1', '3128', 'frank', 'passw0rd', 'Ethernet'),
                    call('192.168.0.1', '3128', 'frank', 'passw0rd', 'Ethernet'),
                    call(['salt.com', 'test.com'], 'Ethernet')
                ]

                set_proxy_mock.assert_has_calls(calls)
                self.assertEqual(out, expected)

    def test_set_proxy_macos_same(self):
        '''
            Test to make sure we can set the proxy settings on macOS
        '''
        with patch.dict(proxy.__grains__, {'os': 'Darwin'}):
            expected = {'changes': {
                },
                'comment': 'http proxy settings already set.\nhttps proxy settings already set.\nftp proxy settings'
                           ' already set.\nProxy bypass domains are already set correctly.\n',
                'name': '192.168.0.1',
                'result': True}

            proxy_val = {
                'enabled': True,
                'server': '192.168.0.1',
                'port': '3128'
            }

            set_proxy_mock = MagicMock()
            patches = {
                'proxy.get_http_proxy': MagicMock(return_value=proxy_val),
                'proxy.get_https_proxy': MagicMock(return_value=proxy_val),
                'proxy.get_ftp_proxy': MagicMock(return_value=proxy_val),
                'proxy.get_proxy_bypass': MagicMock(return_value=['test.com', 'salt.com']),
                'proxy.set_http_proxy': set_proxy_mock,
                'proxy.set_https_proxy': set_proxy_mock,
                'proxy.set_ftp_proxy': set_proxy_mock,
                'proxy.set_proxy_bypass': set_proxy_mock,
            }

            with patch.dict(proxy.__salt__, patches):
                out = proxy.managed('192.168.0.1', '3128', user='frank', password='passw0rd',
                                    bypass_domains=['salt.com', 'test.com'])

                assert not set_proxy_mock.called
                self.assertEqual(out, expected)

    def test_set_proxy_windows(self):
        '''
            Test to make sure we can set the proxy settings on Windows
        '''
        with patch.dict(proxy.__grains__, {'os': 'Windows'}):
            expected = {
                'changes': {},
                'comment': 'Proxy settings updated correctly',
                'name': '192.168.0.1',
                'result': True
            }

            set_proxy_mock = MagicMock(return_value=True)
            patches = {
                'proxy.get_proxy_win': MagicMock(return_value={}),
                'proxy.get_proxy_bypass': MagicMock(return_value=[]),
                'proxy.set_proxy_win': set_proxy_mock,
            }

            with patch.dict(proxy.__salt__, patches):
                out = proxy.managed('192.168.0.1', '3128', user='frank', password='passw0rd',
                                    bypass_domains=['salt.com', 'test.com'])

                set_proxy_mock.assert_called_once_with('192.168.0.1', '3128', ['http', 'https', 'ftp'],
                                                       ['salt.com', 'test.com'])
                self.assertEqual(out, expected)

    def test_set_proxy_windows_same(self):
        '''
            Test to make sure we can set the proxy settings on Windows
        '''
        with patch.dict(proxy.__grains__, {'os': 'Windows'}):
            expected = {
                'changes': {},
                'comment': 'Proxy settings already correct.',
                'name': '192.168.0.1',
                'result': True
            }

            proxy_val = {
                'enabled': True,
                'http': {
                    'enabled': True,
                    'server': '192.168.0.1',
                    'port': '3128'
                },
                'https': {
                    'enabled': True,
                    'server': '192.168.0.1',
                    'port': '3128'
                },
                'ftp': {
                    'enabled': True,
                    'server': '192.168.0.1',
                    'port': '3128'
                }
            }

            set_proxy_mock = MagicMock(return_value=True)
            patches = {
                'proxy.get_proxy_win': MagicMock(return_value=proxy_val),
                'proxy.get_proxy_bypass': MagicMock(return_value=['salt.com', 'test.com']),
                'proxy.set_proxy_win': set_proxy_mock,
            }

            with patch.dict(proxy.__salt__, patches):
                out = proxy.managed('192.168.0.1', '3128', user='frank', password='passw0rd',
                                    bypass_domains=['salt.com', 'test.com'])

                assert not set_proxy_mock.called
                self.assertEqual(out, expected)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)


    ====================================
    Custom Salt TestCase Implementations
    ====================================

    Custom reusable :class:`TestCase<python2:unittest.TestCase>`
    implementations.
'''
# pylint: disable=repr-flag-used-in-string

# Import python libs
from __future__ import absolute_import, unicode_literals
import os
import re
import sys
import time
import stat
import errno
import signal
import textwrap
import logging
import tempfile
import subprocess
from datetime import datetime, timedelta

# Import salt testing libs
from tests.support.unit import TestCase
from tests.support.helpers import (
    RedirectStdStreams, requires_sshd_server, win32_kill_process_tree
)
from tests.support.runtests import RUNTIME_VARS
from tests.support.mixins import AdaptedConfigurationTestCaseMixin, SaltClientTestCaseMixin
from tests.support.paths import ScriptPathMixin, INTEGRATION_TEST_DIR, CODE_DIR, PYEXEC, SCRIPT_DIR

# Import 3rd-party libs
from salt.ext import six
from salt.ext.six.moves import cStringIO  # pylint: disable=import-error

STATE_FUNCTION_RUNNING_RE = re.compile(
    r'''The function (?:"|')(?P<state_func>.*)(?:"|') is running as PID '''
    r'(?P<pid>[\d]+) and was started at (?P<date>.*) with jid (?P<jid>[\d]+)'
)
SCRIPT_TEMPLATES = {
    'salt': [
        'from salt.scripts import salt_main\n',
        'if __name__ == \'__main__\':'
        '    salt_main()'
    ],
    'salt-api': [
        'import salt.cli\n',
        'def main():',
        '    sapi = salt.cli.SaltAPI()',
        '    sapi.run()\n',
        'if __name__ == \'__main__\':',
        '    main()'
    ],
    'common': [
        'from salt.scripts import salt_{0}\n',
        'if __name__ == \'__main__\':',
        '    salt_{0}()'
    ]
}

log = logging.getLogger(__name__)


class ShellTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
    '''
    Execute a test for a shell command
    '''

    def get_script_path(self, script_name):
        '''
        Return the path to a testing runtime script
        '''
        if not os.path.isdir(RUNTIME_VARS.TMP_SCRIPT_DIR):
            os.makedirs(RUNTIME_VARS.TMP_SCRIPT_DIR)

        script_path = os.path.join(RUNTIME_VARS.TMP_SCRIPT_DIR, script_name)
        if not os.path.isfile(script_path):
            log.debug('Generating {0}'.format(script_path))

            # Late import
            import salt.utils.files

            with salt.utils.files.fopen(script_path, 'w') as sfh:
                script_template = SCRIPT_TEMPLATES.get(script_name, None)
                if script_template is None:
                    script_template = SCRIPT_TEMPLATES.get('common', None)
                if script_template is None:
                    raise RuntimeError(
                        '{0} does not know how to handle the {1} script'.format(
                            self.__class__.__name__,
                            script_name
                        )
                    )
                contents = (
                    '#!{0}\n'.format(sys.executable) +
                    '\n'.join(script_template).format(script_name.replace('salt-', ''))
                )
                sfh.write(contents)
                log.debug(
                    'Wrote the following contents to temp script %s:\n%s',
                    script_path, contents
                )
            st = os.stat(script_path)
            os.chmod(script_path, st.st_mode | stat.S_IEXEC)

        return script_path

    def run_salt(self, arg_str, with_retcode=False, catch_stderr=False, timeout=15):
        r'''
        Run the ``salt`` CLI tool with the provided arguments

        .. code-block:: python

            class MatchTest(ShellTestCase):
                def test_list(self):
                    """
                    test salt -L matcher
                    """
                    data = self.run_salt('-L minion test.ping')
                    data = '\n'.join(data)
                    self.assertIn('minion', data)
        '''
        arg_str = '-c {0} -t {1} {2}'.format(self.get_config_dir(), timeout, arg_str)
        return self.run_script('salt', arg_str, with_retcode=with_retcode, catch_stderr=catch_stderr, timeout=timeout)

    def run_ssh(self, arg_str, with_retcode=False, timeout=25,
                catch_stderr=False, wipe=False, raw=False):
        '''
        Execute salt-ssh
        '''
        arg_str = '{0} {1} -c {2} -i --priv {3} --roster-file {4} localhost {5} --out=json'.format(
            ' -W' if wipe else '',
            ' -r' if raw else '',
            self.get_config_dir(),
            os.path.join(RUNTIME_VARS.TMP_CONF_DIR, 'key_test'),
            os.path.join(RUNTIME_VARS.TMP_CONF_DIR, 'roster'),
            arg_str
        )
        return self.run_script('salt-ssh', arg_str, with_retcode=with_retcode, catch_stderr=catch_stderr, raw=True)

    def run_run(self,
                arg_str,
                with_retcode=False,
                catch_stderr=False,
                asynchronous=False,
                timeout=60,
                config_dir=None,
                **kwargs):
        '''
        Execute salt-run
        '''
        asynchronous = kwargs.get('async', asynchronous)
        arg_str = '-c {0}{async_flag} -t {timeout} {1}'.format(
                config_dir or self.get_config_dir(),
                arg_str,
                timeout=timeout,
                async_flag=' --async' if asynchronous else '')
        return self.run_script('salt-run',
                               arg_str,
                               with_retcode=with_retcode,
                               catch_stderr=catch_stderr,
                               timeout=timeout)

    def run_run_plus(self, fun, *arg, **kwargs):
        '''
        Execute the runner function and return the return data and output in a dict
        '''
        ret = {'fun': fun}

        # Late import
        import salt.config
        import salt.output
        import salt.runner
        from salt.ext.six.moves import cStringIO

        opts = salt.config.master_config(
            self.get_config_file_path('master')
        )

        opts_arg = list(arg)
        if kwargs:
            opts_arg.append({'__kwarg__': True})
            opts_arg[-1].update(kwargs)

        opts.update({'doc': False, 'fun': fun, 'arg': opts_arg})
        with RedirectStdStreams():
            runner = salt.runner.Runner(opts)
            ret['return'] = runner.run()
            try:
                ret['jid'] = runner.jid
            except AttributeError:
                ret['jid'] = None

        # Compile output
        # TODO: Support outputters other than nested
        opts['color'] = False
        opts['output_file'] = cStringIO()
        try:
            salt.output.display_output(ret['return'], opts=opts)
            ret['out'] = opts['output_file'].getvalue()
        finally:
            opts['output_file'].close()

        return ret

    def run_key(self, arg_str, catch_stderr=False, with_retcode=False):
        '''
        Execute salt-key
        '''
        arg_str = '-c {0} {1}'.format(self.get_config_dir(), arg_str)
        return self.run_script(
            'salt-key',
            arg_str,
            catch_stderr=catch_stderr,
            with_retcode=with_retcode
        )

    def run_cp(self, arg_str, with_retcode=False, catch_stderr=False):
        '''
        Execute salt-cp
        '''
        arg_str = '--config-dir {0} {1}'.format(self.get_config_dir(), arg_str)
        return self.run_script('salt-cp', arg_str, with_retcode=with_retcode, catch_stderr=catch_stderr)

    def run_call(self, arg_str, with_retcode=False, catch_stderr=False, local=False):
        arg_str = '{0} --config-dir {1} {2}'.format('--local' if local else '',
                                                    self.get_config_dir(), arg_str)

        return self.run_script('salt-call', arg_str, with_retcode=with_retcode, catch_stderr=catch_stderr)

    def run_cloud(self, arg_str, catch_stderr=False, timeout=None):
        '''
        Execute salt-cloud
        '''
        arg_str = '-c {0} {1}'.format(self.get_config_dir(), arg_str)
        return self.run_script('salt-cloud', arg_str, catch_stderr, timeout)

    def run_script(self,
                   script,
                   arg_str,
                   catch_stderr=False,
                   with_retcode=False,
                   catch_timeout=False,
                   # FIXME A timeout of zero or disabling timeouts may not return results!
                   timeout=15,
                   raw=False,
                   popen_kwargs=None,
                   log_output=None):
        '''
        Execute a script with the given argument string

        The ``log_output`` argument is ternary, it can be True, False, or None.
        If the value is boolean, then it forces the results to either be logged
        or not logged. If it is None, then the return code of the subprocess
        determines whether or not to log results.
        '''

        import salt.utils

        script_path = self.get_script_path(script)
        if not os.path.isfile(script_path):
            return False
        popen_kwargs = popen_kwargs or {}

        if salt.utils.platform.is_windows():
            cmd = 'python '
            if 'cwd' not in popen_kwargs:
                popen_kwargs['cwd'] = os.getcwd()
            if 'env' not in popen_kwargs:
                popen_kwargs['env'] = os.environ.copy()
                if sys.version_info[0] < 3:
                    popen_kwargs['env'][b'PYTHONPATH'] = os.getcwd().encode()
                else:
                    popen_kwargs['env']['PYTHONPATH'] = os.getcwd()
        else:
            cmd = 'PYTHONPATH='
            python_path = os.environ.get('PYTHONPATH', None)
            if python_path is not None:
                cmd += '{0}:'.format(python_path)

            if sys.version_info[0] < 3:
                cmd += '{0} '.format(':'.join(sys.path[1:]))
            else:
                cmd += '{0} '.format(':'.join(sys.path[0:]))
            cmd += 'python{0}.{1} '.format(*sys.version_info)
        cmd += '{0} '.format(script_path)
        cmd += '{0} '.format(arg_str)

        tmp_file = tempfile.SpooledTemporaryFile()

        popen_kwargs = dict({
            'shell': True,
            'stdout': tmp_file,
            'universal_newlines': True,
        }, **popen_kwargs)

        if catch_stderr is True:
            popen_kwargs['stderr'] = subprocess.PIPE

        if not sys.platform.lower().startswith('win'):
            popen_kwargs['close_fds'] = True

            def detach_from_parent_group():
                # detach from parent group (no more inherited signals!)
                os.setpgrp()

            popen_kwargs['preexec_fn'] = detach_from_parent_group

        def format_return(retcode, stdout, stderr=None, timed_out=False):
            '''
            DRY helper to log script result if it failed, and then return the
            desired output based on whether or not stderr was desired, and
            wither or not a retcode was desired.
            '''
            log_func = log.debug
            if timed_out:
                log.error(
                    'run_script timed out after %d seconds (process killed)',
                    timeout
                )
                log_func = log.error

            if log_output is True \
                    or timed_out \
                    or (log_output is None and retcode != 0):
                log_func(
                    'run_script results for: %s %s\n'
                    'return code: %s\n'
                    'stdout:\n'
                    '%s\n\n'
                    'stderr:\n'
                    '%s',
                    script, arg_str, retcode, stdout, stderr
                )

            stdout = stdout or ''
            stderr = stderr or ''

            if not raw:
                stdout = stdout.splitlines()
                stderr = stderr.splitlines()

            ret = [stdout]
            if catch_stderr:
                ret.append(stderr)
            if with_retcode:
                ret.append(retcode)
            if catch_timeout:
                ret.append(timed_out)

            return ret[0] if len(ret) == 1 else tuple(ret)

        process = subprocess.Popen(cmd, **popen_kwargs)

        # Late import
        import salt.utils.platform

        if timeout is not None:
            stop_at = datetime.now() + timedelta(seconds=timeout)
            term_sent = False
            while True:
                process.poll()
                time.sleep(0.1)
                if datetime.now() <= stop_at:
                    # We haven't reached the timeout yet
                    if process.returncode is not None:
                        break
                else:
                    # We've reached the timeout
                    if term_sent is False:
                        # Kill the process group since sending the term signal
                        # would only terminate the shell, not the command
                        # executed in the shell
                        if salt.utils.platform.is_windows():
                            _, alive = win32_kill_process_tree(process.pid)
                            if alive:
                                log.error("Child processes still alive: %s", alive)
                        else:
                            os.killpg(os.getpgid(process.pid), signal.SIGINT)
                        term_sent = True
                        continue

                    try:
                        # As a last resort, kill the process group
                        if salt.utils.platform.is_windows():
                            _, alive = win32_kill_process_tree(process.pid)
                            if alive:
                                log.error("Child processes still alive: %s", alive)
                        else:
                            os.killpg(os.getpgid(process.pid), signal.SIGINT)
                    except OSError as exc:
                        if exc.errno != errno.ESRCH:
                            # If errno is not "no such process", raise
                            raise

                    return format_return(
                        process.returncode,
                        *process.communicate(),
                        timed_out=True
                    )

        tmp_file.seek(0)

        if sys.version_info >= (3,):
            try:
                out = tmp_file.read().decode(__salt_system_encoding__)
            except (NameError, UnicodeDecodeError):
                # Let's cross our fingers and hope for the best
                out = tmp_file.read().decode('utf-8')
        else:
            out = tmp_file.read()

        if catch_stderr:
            if sys.version_info < (2, 7):
                # On python 2.6, the subprocess'es communicate() method uses
                # select which, is limited by the OS to 1024 file descriptors
                # We need more available descriptors to run the tests which
                # need the stderr output.
                # So instead of .communicate() we wait for the process to
                # finish, but, as the python docs state "This will deadlock
                # when using stdout=PIPE and/or stderr=PIPE and the child
                # process generates enough output to a pipe such that it
                # blocks waiting for the OS pipe buffer to accept more data.
                # Use communicate() to avoid that." <- a catch, catch situation
                #
                # Use this work around were it's needed only, python 2.6
                process.wait()
                err = process.stderr.read()
            else:
                _, err = process.communicate()
            # Force closing stderr/stdout to release file descriptors
            if process.stdout is not None:
                process.stdout.close()
            if process.stderr is not None:
                process.stderr.close()

            # pylint: disable=maybe-no-member
            try:
                return format_return(process.returncode, out, err or '')
            finally:
                try:
                    if os.path.exists(tmp_file.name):
                        if isinstance(tmp_file.name, six.string_types):
                            # tmp_file.name is an int when using SpooledTemporaryFiles
                            # int types cannot be used with os.remove() in Python 3
                            os.remove(tmp_file.name)
                        else:
                            # Clean up file handles
                            tmp_file.close()
                    process.terminate()
                except OSError as err:
                    # process already terminated
                    pass
            # pylint: enable=maybe-no-member

        # TODO Remove this?
        process.communicate()
        if process.stdout is not None:
            process.stdout.close()

        try:
            return format_return(process.returncode, out)
        finally:
            try:
                if os.path.exists(tmp_file.name):
                    if isinstance(tmp_file.name, six.string_types):
                        # tmp_file.name is an int when using SpooledTemporaryFiles
                        # int types cannot be used with os.remove() in Python 3
                        os.remove(tmp_file.name)
                    else:
                        # Clean up file handles
                        tmp_file.close()
                process.terminate()
            except OSError as err:
                # process already terminated
                pass


class ShellCase(ShellTestCase, AdaptedConfigurationTestCaseMixin, ScriptPathMixin):
    '''
    Execute a test for a shell command
    '''

    _code_dir_ = CODE_DIR
    _script_dir_ = SCRIPT_DIR
    _python_executable_ = PYEXEC
    RUN_TIMEOUT = 500

    def chdir(self, dirname):
        try:
            os.chdir(dirname)
        except OSError:
            os.chdir(INTEGRATION_TEST_DIR)

    def run_salt(self, arg_str, with_retcode=False, catch_stderr=False,  # pylint: disable=W0221
            timeout=RUN_TIMEOUT, popen_kwargs=None):
        '''
        Execute salt
        '''
        arg_str = '-c {0} -t {1} {2}'.format(self.get_config_dir(), timeout, arg_str)
        ret = self.run_script('salt',
                              arg_str,
                              with_retcode=with_retcode,
                              catch_stderr=catch_stderr,
                              timeout=timeout,
                              popen_kwargs=popen_kwargs)
        log.debug('Result of run_salt for command \'%s\': %s', arg_str, ret)
        return ret

    def run_spm(self, arg_str, with_retcode=False, catch_stderr=False, timeout=RUN_TIMEOUT):  # pylint: disable=W0221
        '''
        Execute spm
        '''
        ret = self.run_script('spm',
                              arg_str,
                              with_retcode=with_retcode,
                              catch_stderr=catch_stderr,
                              timeout=timeout)
        log.debug('Result of run_spm for command \'%s\': %s', arg_str, ret)
        return ret

    def run_ssh(self, arg_str, with_retcode=False, catch_stderr=False,  # pylint: disable=W0221
                timeout=RUN_TIMEOUT, wipe=True, raw=False):
        '''
        Execute salt-ssh
        '''
        arg_str = '{0} -ldebug{1} -c {2} -i --priv {3} --roster-file {4} --out=json localhost {5}'.format(
            ' -W' if wipe else '',
            ' -r' if raw else '',
            self.get_config_dir(),
            os.path.join(RUNTIME_VARS.TMP_CONF_DIR, 'key_test'),
            os.path.join(RUNTIME_VARS.TMP_CONF_DIR, 'roster'),
            arg_str)
        ret = self.run_script('salt-ssh',
                              arg_str,
                              with_retcode=with_retcode,
                              catch_stderr=catch_stderr,
                              timeout=timeout,
                              raw=True)
        log.debug('Result of run_ssh for command \'%s\': %s', arg_str, ret)
        return ret

    def run_run(self, arg_str, with_retcode=False, catch_stderr=False,
            asynchronous=False, timeout=RUN_TIMEOUT, config_dir=None, **kwargs):
        '''
        Execute salt-run
        '''
        asynchronous = kwargs.get('async', asynchronous)
        arg_str = '-c {0}{async_flag} -t {timeout} {1}'.format(config_dir or self.get_config_dir(),
                                                               arg_str,
                                                               timeout=timeout,
                                                               async_flag=' --async' if asynchronous else '')
        ret = self.run_script('salt-run',
                              arg_str,
                              with_retcode=with_retcode,
                              catch_stderr=catch_stderr,
                              timeout=timeout + 10)
        log.debug('Result of run_run for command \'%s\': %s', arg_str, ret)
        return ret

    def run_run_plus(self, fun, *arg, **kwargs):
        '''
        Execute the runner function and return the return data and output in a dict
        '''
        # Late import
        import salt.runner
        import salt.output
        ret = {'fun': fun}
        from_scratch = bool(kwargs.pop('__reload_config', False))
        # Have to create an empty dict and then update it, as the result from
        # self.get_config() is an ImmutableDict which cannot be updated.
        opts = {}
        opts.update(self.get_config('client_config', from_scratch=from_scratch))
        opts_arg = list(arg)
        if kwargs:
            opts_arg.append({'__kwarg__': True})
            opts_arg[-1].update(kwargs)
        opts.update({'doc': False, 'fun': fun, 'arg': opts_arg})
        with RedirectStdStreams():
            runner = salt.runner.Runner(opts)
            ret['return'] = runner.run()
            try:
                ret['jid'] = runner.jid
            except AttributeError:
                ret['jid'] = None

        # Compile output
        # TODO: Support outputters other than nested
        opts['color'] = False
        opts['output_file'] = cStringIO()
        try:
            salt.output.display_output(ret['return'], opts=opts)
            ret['out'] = opts['output_file'].getvalue().splitlines()
        finally:
            opts['output_file'].close()

        log.debug('Result of run_run_plus for fun \'%s\' with arg \'%s\': %s',
                  fun, opts_arg, ret)
        return ret

    def run_key(self, arg_str, catch_stderr=False, with_retcode=False,  # pylint: disable=W0221
            timeout=RUN_TIMEOUT):
        '''
        Execute salt-key
        '''
        arg_str = '-c {0} {1}'.format(self.get_config_dir(), arg_str)
        ret = self.run_script('salt-key',
                              arg_str,
                              catch_stderr=catch_stderr,
                              with_retcode=with_retcode,
                              timeout=timeout)
        log.debug('Result of run_key for command \'%s\': %s', arg_str, ret)
        return ret

    def run_cp(self, arg_str, with_retcode=False, catch_stderr=False,  # pylint: disable=W0221
            timeout=RUN_TIMEOUT):
        '''
        Execute salt-cp
        '''
        # Note: not logging result of run_cp because it will log a bunch of
        # bytes which will not be very helpful.
        arg_str = '--config-dir {0} {1}'.format(self.get_config_dir(), arg_str)
        return self.run_script('salt-cp',
                               arg_str,
                               with_retcode=with_retcode,
                               catch_stderr=catch_stderr,
                               timeout=timeout)

    def run_call(self, arg_str, with_retcode=False, catch_stderr=False,  # pylint: disable=W0221
            local=False, timeout=RUN_TIMEOUT):
        '''
        Execute salt-call.
        '''
        arg_str = '{0} --config-dir {1} {2}'.format('--local' if local else '',
                                                    self.get_config_dir(), arg_str)
        ret = self.run_script('salt-call',
                              arg_str,
                              with_retcode=with_retcode,
                              catch_stderr=catch_stderr,
                              timeout=timeout)
        log.debug('Result of run_call for command \'%s\': %s', arg_str, ret)
        return ret

    def run_cloud(self, arg_str, catch_stderr=False, timeout=RUN_TIMEOUT):
        '''
        Execute salt-cloud
        '''
        arg_str = '-c {0} {1}'.format(self.get_config_dir(), arg_str)
        ret = self.run_script('salt-cloud',
                              arg_str,
                              catch_stderr,
                              timeout=timeout)
        log.debug('Result of run_cloud for command \'%s\': %s', arg_str, ret)
        return ret


class SPMTestUserInterface(object):
    '''
    Test user interface to SPMClient
    '''
    def __init__(self):
        self._status = []
        self._confirm = []
        self._error = []

    def status(self, msg):
        self._status.append(msg)

    def confirm(self, action):
        self._confirm.append(action)

    def error(self, msg):
        self._error.append(msg)


class SPMCase(TestCase, AdaptedConfigurationTestCaseMixin):
    '''
    Class for handling spm commands
    '''

    def _spm_build_files(self, config):
        self.formula_dir = os.path.join(' '.join(config['file_roots']['base']), 'formulas')
        self.formula_sls_dir = os.path.join(self.formula_dir, 'apache')
        self.formula_sls = os.path.join(self.formula_sls_dir, 'apache.sls')
        self.formula_file = os.path.join(self.formula_dir, 'FORMULA')

        dirs = [self.formula_dir, self.formula_sls_dir]
        for f_dir in dirs:
            os.makedirs(f_dir)

        # Late import
        import salt.utils.files

        with salt.utils.files.fopen(self.formula_sls, 'w') as fp:
            fp.write(textwrap.dedent('''\
                     install-apache:
                       pkg.installed:
                         - name: apache2
                     '''))

        with salt.utils.files.fopen(self.formula_file, 'w') as fp:
            fp.write(textwrap.dedent('''\
                     name: apache
                     os: RedHat, Debian, Ubuntu, Suse, FreeBSD
                     os_family: RedHat, Debian, Suse, FreeBSD
                     version: 201506
                     release: 2
                     summary: Formula for installing Apache
                     description: Formula for installing Apache
                     '''))

    def _spm_config(self, assume_yes=True):
        self._tmp_spm = tempfile.mkdtemp()
        config = self.get_temp_config('minion', **{
            'spm_logfile': os.path.join(self._tmp_spm, 'log'),
            'spm_repos_config': os.path.join(self._tmp_spm, 'etc', 'spm.repos'),
            'spm_cache_dir': os.path.join(self._tmp_spm, 'cache'),
            'spm_build_dir': os.path.join(self._tmp_spm, 'build'),
            'spm_build_exclude': ['apache/.git'],
            'spm_db_provider': 'sqlite3',
            'spm_files_provider': 'local',
            'spm_db': os.path.join(self._tmp_spm, 'packages.db'),
            'extension_modules': os.path.join(self._tmp_spm, 'modules'),
            'file_roots': {'base': [self._tmp_spm, ]},
            'formula_path': os.path.join(self._tmp_spm, 'salt'),
            'pillar_path': os.path.join(self._tmp_spm, 'pillar'),
            'reactor_path': os.path.join(self._tmp_spm, 'reactor'),
            'assume_yes': True if assume_yes else False,
            'force': False,
            'verbose': False,
            'cache': 'localfs',
            'cachedir': os.path.join(self._tmp_spm, 'cache'),
            'spm_repo_dups': 'ignore',
            'spm_share_dir': os.path.join(self._tmp_spm, 'share'),
        })

        import salt.utils.files
        import salt.utils.yaml

        if not os.path.isdir(config['formula_path']):
            os.makedirs(config['formula_path'])

        with salt.utils.files.fopen(os.path.join(self._tmp_spm, 'spm'), 'w') as fp:
            salt.utils.yaml.safe_dump(config, fp)

        return config

    def _spm_create_update_repo(self, config):

        build_spm = self.run_spm('build', self.config, self.formula_dir)

        c_repo = self.run_spm('create_repo', self.config,
                              self.config['spm_build_dir'])

        repo_conf_dir = self.config['spm_repos_config'] + '.d'
        os.makedirs(repo_conf_dir)

        # Late import
        import salt.utils.files

        with salt.utils.files.fopen(os.path.join(repo_conf_dir, 'spm.repo'), 'w') as fp:
            fp.write(textwrap.dedent('''\
                     local_repo:
                       url: file://{0}
                     '''.format(self.config['spm_build_dir'])))

        u_repo = self.run_spm('update_repo', self.config)

    def _spm_client(self, config):
        import salt.spm
        self.ui = SPMTestUserInterface()
        client = salt.spm.SPMClient(self.ui, config)
        return client

    def run_spm(self, cmd, config, arg=None):
        client = self._spm_client(config)
        spm_cmd = client.run([cmd, arg])
        client._close()
        return self.ui._status


class ModuleCase(TestCase, SaltClientTestCaseMixin):
    '''
    Execute a module function
    '''

    def minion_run(self, _function, *args, **kw):
        '''
        Run a single salt function on the 'minion' target and condition
        the return down to match the behavior of the raw function call
        '''
        return self.run_function(_function, args, **kw)

    def run_function(self, function, arg=(), minion_tgt='minion', timeout=300, **kwargs):
        '''
        Run a single salt function and condition the return down to match the
        behavior of the raw function call
        '''
        know_to_return_none = (
            'file.chown', 'file.chgrp', 'ssh.recv_known_host_entries'
        )
        if 'f_arg' in kwargs:
            kwargs['arg'] = kwargs.pop('f_arg')
        if 'f_timeout' in kwargs:
            kwargs['timeout'] = kwargs.pop('f_timeout')
        orig = self.client.cmd(minion_tgt,
                               function,
                               arg,
                               timeout=timeout,
                               kwarg=kwargs)

        if minion_tgt not in orig:
            self.skipTest(
                'WARNING(SHOULD NOT HAPPEN #1935): Failed to get a reply '
                'from the minion \'{0}\'. Command output: {1}'.format(
                    minion_tgt, orig
                )
            )
        elif orig[minion_tgt] is None and function not in know_to_return_none:
            self.skipTest(
                'WARNING(SHOULD NOT HAPPEN #1935): Failed to get \'{0}\' from '
                'the minion \'{1}\'. Command output: {2}'.format(
                    function, minion_tgt, orig
                )
            )

        # Try to match stalled state functions
        orig[minion_tgt] = self._check_state_return(orig[minion_tgt])

        return orig[minion_tgt]

    def run_state(self, function, **kwargs):
        '''
        Run the state.single command and return the state return structure
        '''
        ret = self.run_function('state.single', [function], **kwargs)
        return self._check_state_return(ret)

    def _check_state_return(self, ret):
        if isinstance(ret, dict):
            # This is the supposed return format for state calls
            return ret

        if isinstance(ret, list):
            jids = []
            # These are usually errors
            for item in ret[:]:
                if not isinstance(item, six.string_types):
                    # We don't know how to handle this
                    continue
                match = STATE_FUNCTION_RUNNING_RE.match(item)
                if not match:
                    # We don't know how to handle this
                    continue
                jid = match.group('jid')
                if jid in jids:
                    continue

                jids.append(jid)

                job_data = self.run_function('saltutil.find_job', [jid])
                job_kill = self.run_function('saltutil.kill_job', [jid])
                msg = (
                    'A running state.single was found causing a state lock. '
                    'Job details: \'{0}\'  Killing Job Returned: \'{1}\''.format(
                        job_data, job_kill
                    )
                )
                ret.append('[TEST SUITE ENFORCED]{0}'
                           '[/TEST SUITE ENFORCED]'.format(msg))
        return ret


class SyndicCase(TestCase, SaltClientTestCaseMixin):
    '''
    Execute a syndic based execution test
    '''
    _salt_client_config_file_name_ = 'syndic_master'

    def run_function(self, function, arg=()):
        '''
        Run a single salt function and condition the return down to match the
        behavior of the raw function call
        '''
        orig = self.client.cmd('minion', function, arg, timeout=25)
        if 'minion' not in orig:
            self.skipTest(
                'WARNING(SHOULD NOT HAPPEN #1935): Failed to get a reply '
                'from the minion. Command output: {0}'.format(orig)
            )
        return orig['minion']


@requires_sshd_server
class SSHCase(ShellCase):
    '''
    Execute a command via salt-ssh
    '''
    def _arg_str(self, function, arg):
        return '{0} {1}'.format(function, ' '.join(arg))

    def run_function(self, function, arg=(), timeout=180, wipe=True, raw=False, **kwargs):
        '''
        We use a 180s timeout here, which some slower systems do end up needing
        '''
        ret = self.run_ssh(self._arg_str(function, arg), timeout=timeout,
                           wipe=wipe, raw=raw)
        log.debug('SSHCase run_function executed %s with arg %s', function, arg)
        log.debug('SSHCase JSON return: %s', ret)

        # Late import
        import salt.utils.json

        try:
            return salt.utils.json.loads(ret)['localhost']
        except Exception:
            return ret


class ClientCase(AdaptedConfigurationTestCaseMixin, TestCase):
    '''
    A base class containing relevant options for starting the various Salt
    Python API entrypoints
    '''
    def get_opts(self):
        # Late import
        import salt.config

        return salt.config.client_config(self.get_config_file_path('master'))

    def mkdir_p(self, path):
        try:
            os.makedirs(path)
        except OSError as exc:  # Python >2.5
            if exc.errno == errno.EEXIST and os.path.isdir(path):
                pass
            else:
                raise


# ----- Backwards Compatible Imports -------------------------------------------------------------------------------->
from tests.support.mixins import ShellCaseCommonTestsMixin  # pylint: disable=unused-import
# <---- Backwards Compatible Imports ---------------------------------------------------------------------------------



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    Mock,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.utils.network
import salt.modules.win_network as win_network

try:
    import wmi
    HAS_WMI = True
except ImportError:
    HAS_WMI = False


class Mockwmi(object):
    '''
    Mock wmi class
    '''
    NetConnectionID = 'Ethernet'

    def __init__(self):
        pass


class Mockwinapi(object):
    '''
    Mock winapi class
    '''
    def __init__(self):
        pass

    class winapi(object):
        '''
        Mock winapi class
        '''
        def __init__(self):
            pass

        class Com(object):
            '''
            Mock Com method
            '''
            def __enter__(self):
                return self

            def __exit__(self, *exc_info):
                return False


@skipIf(NO_MOCK, NO_MOCK_REASON)
class WinNetworkTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.win_network
    '''
    def setup_loader_modules(self):
        self.WMI = Mock()
        self.addCleanup(delattr, self, 'WMI')
        return {win_network: {}}

    # 'ping' function tests: 1

    def test_ping(self):
        '''
        Test if it performs a ping to a host.
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(win_network.__salt__, {'cmd.run': mock}):
            self.assertTrue(win_network.ping('127.0.0.1'))

    # 'netstat' function tests: 1

    def test_netstat(self):
        '''
        Test if it return information on open ports and states
        '''
        ret = ('  Proto  Local Address    Foreign Address    State    PID\n'
               '  TCP    127.0.0.1:1434    0.0.0.0:0    LISTENING    1728\n'
               '  UDP    127.0.0.1:1900    *:*        4240')
        mock = MagicMock(return_value=ret)
        with patch.dict(win_network.__salt__, {'cmd.run': mock}):
            self.assertListEqual(win_network.netstat(),
                                 [{'local-address': '127.0.0.1:1434',
                                   'program': '1728', 'proto': 'TCP',
                                   'remote-address': '0.0.0.0:0',
                                   'state': 'LISTENING'},
                                  {'local-address': '127.0.0.1:1900',
                                   'program': '4240', 'proto': 'UDP',
                                   'remote-address': '*:*', 'state': None}])

    # 'traceroute' function tests: 1

    def test_traceroute(self):
        '''
        Test if it performs a traceroute to a 3rd party host
        '''
        ret = ('  1     1 ms    <1 ms    <1 ms  172.27.104.1\n'
               '  2     1 ms    <1 ms     1 ms  121.242.35.1.s[121.242.35.1]\n'
               '  3     3 ms     2 ms     2 ms  121.242.4.53.s[121.242.4.53]\n')
        mock = MagicMock(return_value=ret)
        with patch.dict(win_network.__salt__, {'cmd.run': mock}):
            self.assertListEqual(win_network.traceroute('google.com'),
                                 [{'count': '1', 'hostname': None,
                                   'ip': '172.27.104.1', 'ms1': '1',
                                   'ms2': '<1', 'ms3': '<1'},
                                  {'count': '2', 'hostname': None,
                                   'ip': '121.242.35.1.s[121.242.35.1]',
                                   'ms1': '1', 'ms2': '<1', 'ms3': '1'},
                                  {'count': '3', 'hostname': None,
                                   'ip': '121.242.4.53.s[121.242.4.53]',
                                   'ms1': '3', 'ms2': '2', 'ms3': '2'}])

    # 'nslookup' function tests: 1

    def test_nslookup(self):
        '''
        Test if it query DNS for information about a domain or ip address
        '''
        ret = ('Server:  ct-dc-3-2.cybage.com\n'
               'Address:  172.27.172.12\n'
               'Non-authoritative answer:\n'
               'Name:    google.com\n'
               'Addresses:  2404:6800:4007:806::200e\n'
               '216.58.196.110\n')
        mock = MagicMock(return_value=ret)
        with patch.dict(win_network.__salt__, {'cmd.run': mock}):
            self.assertListEqual(win_network.nslookup('google.com'),
                                 [{'Server': 'ct-dc-3-2.cybage.com'},
                                  {'Address': '172.27.172.12'},
                                  {'Name': 'google.com'},
                                  {'Addresses': ['2404:6800:4007:806::200e',
                                                 '216.58.196.110']}])

    # 'dig' function tests: 1

    def test_dig(self):
        '''
        Test if it performs a DNS lookup with dig
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(win_network.__salt__, {'cmd.run': mock}):
            self.assertTrue(win_network.dig('google.com'))

    # 'interfaces_names' function tests: 1

    @skipIf(not HAS_WMI, "WMI only available on Windows")
    def test_interfaces_names(self):
        '''
        Test if it return a list of all the interfaces names
        '''
        self.WMI.Win32_NetworkAdapter = MagicMock(return_value=Mockwmi)
        with patch('salt.utils.winapi.Com', MagicMock()), \
                patch.object(self.WMI, 'Win32_NetworkAdapter',
                             return_value=[Mockwmi()]), \
                patch('salt.utils', Mockwinapi), \
                patch.object(wmi, 'WMI', Mock(return_value=self.WMI)):
            self.assertListEqual(win_network.interfaces_names(),
                                 ['Ethernet'])

    # 'interfaces' function tests: 1

    def test_interfaces(self):
        '''
        Test if it return information about all the interfaces on the minion
        '''
        with patch.object(salt.utils.network, 'win_interfaces',
                          MagicMock(return_value=True)):
            self.assertTrue(win_network.interfaces())

    # 'hw_addr' function tests: 1

    def test_hw_addr(self):
        '''
        Test if it return the hardware address (a.k.a. MAC address)
        for a given interface
        '''
        with patch.object(salt.utils.network, 'hw_addr',
                          MagicMock(return_value='Ethernet')):
            self.assertEqual(win_network.hw_addr('Ethernet'), 'Ethernet')

    # 'subnets' function tests: 1

    def test_subnets(self):
        '''
        Test if it returns a list of subnets to which the host belongs
        '''
        with patch.object(salt.utils.network, 'subnets',
                          MagicMock(return_value='10.1.1.0/24')):
            self.assertEqual(win_network.subnets(), '10.1.1.0/24')

    # 'in_subnet' function tests: 1

    def test_in_subnet(self):
        '''
        Test if it returns True if host is within specified subnet,
        otherwise False
        '''
        with patch.object(salt.utils.network, 'in_subnet',
                          MagicMock(return_value=True)):
            self.assertTrue(win_network.in_subnet('10.1.1.0/16'))

    # 'get_route' function tests: 1

    def test_get_route(self):
        '''
        Test if it return information on open ports and states
        '''
        ret = ('\n\n'
               'IPAddress         : 10.0.0.15\n'
               'InterfaceIndex    : 3\n'
               'InterfaceAlias    : Wi-Fi\n'
               'AddressFamily     : IPv4\n'
               'Type              : Unicast\n'
               'PrefixLength      : 24\n'
               'PrefixOrigin      : Dhcp\n'
               'SuffixOrigin      : Dhcp\n'
               'AddressState      : Preferred\n'
               'ValidLifetime     : 6.17:52:39\n'
               'PreferredLifetime : 6.17:52:39\n'
               'SkipAsSource      : False\n'
               'PolicyStore       : ActiveStore\n'
               '\n\n'
               'Caption            :\n'
               'Description        :\n'
               'ElementName        :\n'
               'InstanceID         : :8:8:8:9:55=55;:8;8;:8;55;\n'
               'AdminDistance      :\n'
               'DestinationAddress :\n'
               'IsStatic           :\n'
               'RouteMetric        : 0\n'
               'TypeOfRoute        : 3\n'
               'AddressFamily      : IPv4\n'
               'CompartmentId      : 1\n'
               'DestinationPrefix  : 0.0.0.0/0\n'
               'InterfaceAlias     : Wi-Fi\n'
               'InterfaceIndex     : 3\n'
               'NextHop            : 10.0.0.1\n'
               'PreferredLifetime  : 6.23:14:43\n'
               'Protocol           : NetMgmt\n'
               'Publish            : No\n'
               'Store              : ActiveStore\n'
               'ValidLifetime      : 6.23:14:43\n'
               'PSComputerName     :\n'
               'ifIndex            : 3')
        mock = MagicMock(return_value=ret)
        with patch.dict(win_network.__salt__, {'cmd.run': mock}):
            self.assertDictEqual(win_network.get_route('192.0.0.8'),
                                 {'destination': '192.0.0.8',
                                  'gateway': '10.0.0.1',
                                  'interface': 'Wi-Fi',
                                  'source': '10.0.0.15'})



# -*- coding: utf-8 -*-
'''
    :codeauthor: Dave Rawks (dave@pandora.com)


    tests.unit.modules.parted_test
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'''

# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin

from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, MagicMock, patch

# Import Salt libs
from salt.exceptions import CommandExecutionError
import salt.modules.parted as parted


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PartedTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        self.cmdrun = MagicMock()
        self.cmdrun_stdout = MagicMock()
        self.addCleanup(delattr, self, 'cmdrun')
        self.addCleanup(delattr, self, 'cmdrun_stdout')
        return {
            parted: {
                '__salt__': {
                    'cmd.run': self.cmdrun,
                    'cmd.run_stdout': self.cmdrun_stdout
                }
            }
        }

    # Test __virtual__ function for module registration

    def test_virtual_bails_on_windows(self):
        '''
        If running windows, __virtual__ shouldn't register module
        '''
        with patch('salt.utils.platform.is_windows', lambda: True):
            ret = parted.__virtual__()
            err = (False, 'The parted execution module failed to load Windows systems are not supported.')
            self.assertEqual(err, ret)

    def test_virtual_bails_without_parted(self):
        '''
        If parted not in PATH, __virtual__ shouldn't register module
        '''
        with patch('salt.utils.path.which', lambda exe: not exe == "parted"),\
                patch('salt.utils.platform.is_windows', return_value=False):
            ret = parted.__virtual__()
            err = (False, 'The parted execution module failed to load parted binary is not in the path.')
            self.assertEqual(err, ret)

    def test_virtual_bails_without_lsblk(self):
        '''
        If lsblk not in PATH, __virtual__ shouldn't register module
        '''
        with patch('salt.utils.path.which', lambda exe: not exe == "lsblk"),\
                patch('salt.utils.platform.is_windows', return_value=False):
            ret = parted.__virtual__()
            err = (False, 'The parted execution module failed to load lsblk binary is not in the path.')
            self.assertEqual(err, ret)

    def test_virtual_bails_without_partprobe(self):
        '''
        If partprobe not in PATH, __virtual__ shouldn't register module
        '''
        with patch('salt.utils.path.which', lambda exe: not exe == "partprobe"),\
                patch('salt.utils.platform.is_windows', return_value=False):
            ret = parted.__virtual__()
            err = (False, 'The parted execution module failed to load partprobe binary is not in the path.')
            self.assertEqual(err, ret)

    def test_virtual(self):
        '''
        On expected platform with correct utils in PATH, register "partition" module
        '''
        with patch('salt.utils.platform.is_windows', lambda: False), \
                patch('salt.utils.path.which', lambda exe: exe in ('parted', 'lsblk', 'partprobe')):
            ret = parted.__virtual__()
            expect = 'partition'
            self.assertEqual(ret, expect)

    # Test probe function

    def test_probe_wo_args(self):
        parted.probe()
        self.cmdrun.assert_called_once_with('partprobe -- ')

    def test_probe_w_single_arg(self):
        with patch('salt.modules.parted._validate_device', MagicMock()):
            parted.probe('/dev/sda')
            self.cmdrun.assert_called_once_with('partprobe -- /dev/sda')

    def test_probe_w_multiple_args(self):
        with patch('salt.modules.parted._validate_device', MagicMock()):
            parted.probe('/dev/sda', '/dev/sdb')
            self.cmdrun.assert_called_once_with('partprobe -- /dev/sda /dev/sdb')

    # Test _list function

    @staticmethod
    def parted_print_output(k):
        output = {
            "valid": (
                '''BYT;\n'''
                '''/dev/sda:4000GB:scsi:512:512:gpt:AMCC 9650SE-24M DISK:;\n'''
                '''1:17.4kB:150MB:150MB:ext3::boot;\n'''
                '''2:3921GB:4000GB:79.3GB:linux-swap(v1)::;\n'''
            ),
            "valid_legacy": (
                '''BYT;\n'''
                '''/dev/sda:4000GB:scsi:512:512:gpt:AMCC 9650SE-24M DISK;\n'''
                '''1:17.4kB:150MB:150MB:ext3::boot;\n'''
                '''2:3921GB:4000GB:79.3GB:linux-swap(v1)::;\n'''
            ),
            "empty": '',
            "bad_label_info": (
                '''BYT;\n'''
                '''badbadbadbad\n'''
                '''1:17.4kB:150MB:150MB:ext3::boot;\n'''
                '''2:3921GB:4000GB:79.3GB:linux-swap(v1)::;\n'''
            ),
            "bad_header": (
                '''badbadbadbad\n'''
                '''/dev/sda:4000GB:scsi:512:512:gpt:AMCC 9650SE-24M DISK:;\n'''
                '''1:17.4kB:150MB:150MB:ext3::boot;\n'''
                '''2:3921GB:4000GB:79.3GB:linux-swap(v1)::;\n'''
            ),
            "bad_partition": (
                '''BYT;\n'''
                '''/dev/sda:4000GB:scsi:512:512:gpt:AMCC 9650SE-24M DISK:;\n'''
                '''badbadbadbad\n'''
                '''2:3921GB:4000GB:79.3GB:linux-swap(v1)::;\n'''
            ),
        }
        return output[k]

    def test_list__without_device(self):
        self.assertRaises(TypeError, parted.list_)

    def test_list__empty_cmd_output(self):
        with patch('salt.modules.parted._validate_device', MagicMock()):
            self.cmdrun_stdout.return_value = self.parted_print_output('empty')
            output = parted.list_('/dev/sda')
            self.cmdrun_stdout.assert_called_once_with('parted -m -s /dev/sda print')
            expected = {'info': {}, 'partitions': {}}
            self.assertEqual(output, expected)

    def test_list__valid_unit_empty_cmd_output(self):
        with patch('salt.modules.parted._validate_device', MagicMock()):
            self.cmdrun_stdout.return_value = self.parted_print_output('empty')
            output = parted.list_('/dev/sda', unit='s')
            self.cmdrun_stdout.assert_called_once_with('parted -m -s /dev/sda unit s print')
            expected = {'info': {}, 'partitions': {}}
            self.assertEqual(output, expected)

    def test_list__invalid_unit(self):
        self.assertRaises(CommandExecutionError, parted.list_, '/dev/sda',
                          unit='badbadbad')
        self.assertFalse(self.cmdrun.called)

    def test_list__bad_header(self):
        with patch('salt.modules.parted._validate_device', MagicMock()):
            self.cmdrun_stdout.return_value = self.parted_print_output('bad_header')
            self.assertRaises(CommandExecutionError, parted.list_, '/dev/sda')
            self.cmdrun_stdout.assert_called_once_with('parted -m -s /dev/sda print')

    def test_list__bad_label_info(self):
        with patch('salt.modules.parted._validate_device', MagicMock()):
            self.cmdrun_stdout.return_value = self.parted_print_output('bad_label_info')
            self.assertRaises(CommandExecutionError, parted.list_, '/dev/sda')
            self.cmdrun_stdout.assert_called_once_with('parted -m -s /dev/sda print')

    def test_list__bad_partition(self):
        with patch('salt.modules.parted._validate_device', MagicMock()):
            self.cmdrun_stdout.return_value = self.parted_print_output('bad_partition')
            self.assertRaises(CommandExecutionError, parted.list_, '/dev/sda')
            self.cmdrun_stdout.assert_called_once_with('parted -m -s /dev/sda print')

    def test_list__valid_cmd_output(self):
        with patch('salt.modules.parted._validate_device', MagicMock()):
            self.cmdrun_stdout.return_value = self.parted_print_output('valid')
            output = parted.list_('/dev/sda')
            self.cmdrun_stdout.assert_called_once_with('parted -m -s /dev/sda print')
            expected = {
                'info': {
                    'logical sector': '512',
                    'physical sector': '512',
                    'interface': 'scsi',
                    'model': 'AMCC 9650SE-24M DISK',
                    'disk': '/dev/sda',
                    'disk flags': '',
                    'partition table': 'gpt',
                    'size': '4000GB'
                },
                'partitions': {
                    '1': {
                        'end': '150MB',
                        'number': '1',
                        'start': '17.4kB',
                        'file system': '',
                        'flags': 'boot',
                        'type': 'ext3',
                        'size': '150MB'},
                    '2': {
                        'end': '4000GB',
                        'number': '2',
                        'start': '3921GB',
                        'file system': '',
                        'flags': '',
                        'type': 'linux-swap(v1)',
                        'size': '79.3GB'
                    }
                }
            }
            self.assertEqual(output, expected)

    def test_list__valid_unit_valid_cmd_output(self):
        with patch('salt.modules.parted._validate_device', MagicMock()):
            self.cmdrun_stdout.return_value = self.parted_print_output('valid')
            output = parted.list_('/dev/sda', unit='s')
            self.cmdrun_stdout.assert_called_once_with('parted -m -s /dev/sda unit s print')
            expected = {
                'info': {
                    'logical sector': '512',
                    'physical sector': '512',
                    'interface': 'scsi',
                    'model': 'AMCC 9650SE-24M DISK',
                    'disk': '/dev/sda',
                    'disk flags': '',
                    'partition table': 'gpt',
                    'size': '4000GB'
                },
                'partitions': {
                    '1': {
                        'end': '150MB',
                        'number': '1',
                        'start': '17.4kB',
                        'file system': '',
                        'flags': 'boot',
                        'type': 'ext3',
                        'size': '150MB'},
                    '2': {
                        'end': '4000GB',
                        'number': '2',
                        'start': '3921GB',
                        'file system': '',
                        'flags': '',
                        'type': 'linux-swap(v1)',
                        'size': '79.3GB'
                    }
                }
            }
            self.assertEqual(output, expected)

    def test_list__valid_legacy_cmd_output(self):
        with patch('salt.modules.parted._validate_device', MagicMock()):
            self.cmdrun_stdout.return_value = self.parted_print_output('valid_legacy')
            output = parted.list_('/dev/sda')
            self.cmdrun_stdout.assert_called_once_with('parted -m -s /dev/sda print')
            expected = {
                'info': {
                    'logical sector': '512',
                    'physical sector': '512',
                    'interface': 'scsi',
                    'model': 'AMCC 9650SE-24M DISK',
                    'disk': '/dev/sda',
                    'partition table': 'gpt',
                    'size': '4000GB'
                },
                'partitions': {
                    '1': {
                        'end': '150MB',
                        'number': '1',
                        'start': '17.4kB',
                        'file system': '',
                        'flags': 'boot',
                        'type': 'ext3',
                        'size': '150MB'},
                    '2': {
                        'end': '4000GB',
                        'number': '2',
                        'start': '3921GB',
                        'file system': '',
                        'flags': '',
                        'type': 'linux-swap(v1)',
                        'size': '79.3GB'
                    }
                }
            }
            self.assertEqual(output, expected)

    def test_list__valid_unit_valid_legacy_cmd_output(self):
        with patch('salt.modules.parted._validate_device', MagicMock()):
            self.cmdrun_stdout.return_value = self.parted_print_output('valid_legacy')
            output = parted.list_('/dev/sda', unit='s')
            self.cmdrun_stdout.assert_called_once_with('parted -m -s /dev/sda unit s print')
            expected = {
                'info': {
                    'logical sector': '512',
                    'physical sector': '512',
                    'interface': 'scsi',
                    'model': 'AMCC 9650SE-24M DISK',
                    'disk': '/dev/sda',
                    'partition table': 'gpt',
                    'size': '4000GB'
                },
                'partitions': {
                    '1': {
                        'end': '150MB',
                        'number': '1',
                        'start': '17.4kB',
                        'file system': '',
                        'flags': 'boot',
                        'type': 'ext3',
                        'size': '150MB'},
                    '2': {
                        'end': '4000GB',
                        'number': '2',
                        'start': '3921GB',
                        'file system': '',
                        'flags': '',
                        'type': 'linux-swap(v1)',
                        'size': '79.3GB'
                    }
                }
            }
            self.assertEqual(output, expected)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jochen Breuer <jbreuer@suse.de>
'''

# Import Python Libs
from __future__ import absolute_import

from contextlib import contextmanager

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    Mock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

from salt.modules import kubernetes


@contextmanager
def mock_kubernetes_library():
    """
    After fixing the bug in 1c821c0e77de58892c77d8e55386fac25e518c31,
    it caused kubernetes._cleanup() to get called for virtually every
    test, which blows up. This prevents that specific blow-up once
    """
    with patch('salt.modules.kubernetes.kubernetes') as mock_kubernetes_lib:
        mock_kubernetes_lib.client.configuration.ssl_ca_cert = ''
        mock_kubernetes_lib.client.configuration.cert_file = ''
        mock_kubernetes_lib.client.configuration.key_file = ''
        yield mock_kubernetes_lib


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not kubernetes.HAS_LIBS, "Kubernetes client lib is not installed. "
                                 "Skipping test_kubernetes.py")
class KubernetesTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.kubernetes
    '''

    def setup_loader_modules(self):
        return {
            kubernetes: {
                '__salt__': {},
            }
        }

    def test_nodes(self):
        '''
        Test node listing.
        :return:
        '''
        with mock_kubernetes_library() as mock_kubernetes_lib:
            with patch.dict(kubernetes.__salt__, {'config.option': Mock(return_value="")}):
                mock_kubernetes_lib.client.CoreV1Api.return_value = Mock(
                    **{"list_node.return_value.to_dict.return_value":
                        {'items': [{'metadata': {'name': 'mock_node_name'}}]}}
                )
                self.assertEqual(kubernetes.nodes(), ['mock_node_name'])
                self.assertTrue(kubernetes.kubernetes.client.CoreV1Api().list_node().to_dict.called)

    def test_deployments(self):
        '''
        Tests deployment listing.
        :return:
        '''
        with mock_kubernetes_library() as mock_kubernetes_lib:
            with patch.dict(kubernetes.__salt__, {'config.option': Mock(return_value="")}):
                mock_kubernetes_lib.client.ExtensionsV1beta1Api.return_value = Mock(
                    **{"list_namespaced_deployment.return_value.to_dict.return_value":
                        {'items': [{'metadata': {'name': 'mock_deployment_name'}}]}}
                )
                self.assertEqual(kubernetes.deployments(), ['mock_deployment_name'])
                self.assertTrue(
                    kubernetes.kubernetes.client.ExtensionsV1beta1Api().list_namespaced_deployment().to_dict.called)

    def test_services(self):
        '''
        Tests services listing.
        :return:
        '''
        with mock_kubernetes_library() as mock_kubernetes_lib:
            with patch.dict(kubernetes.__salt__, {'config.option': Mock(return_value="")}):
                mock_kubernetes_lib.client.CoreV1Api.return_value = Mock(
                    **{"list_namespaced_service.return_value.to_dict.return_value":
                        {'items': [{'metadata': {'name': 'mock_service_name'}}]}}
                )
                self.assertEqual(kubernetes.services(), ['mock_service_name'])
                self.assertTrue(kubernetes.kubernetes.client.CoreV1Api().list_namespaced_service().to_dict.called)

    def test_pods(self):
        '''
        Tests pods listing.
        :return:
        '''
        with mock_kubernetes_library() as mock_kubernetes_lib:
            with patch.dict(kubernetes.__salt__, {'config.option': Mock(return_value="")}):
                mock_kubernetes_lib.client.CoreV1Api.return_value = Mock(
                    **{"list_namespaced_pod.return_value.to_dict.return_value":
                        {'items': [{'metadata': {'name': 'mock_pod_name'}}]}}
                )
                self.assertEqual(kubernetes.pods(), ['mock_pod_name'])
                self.assertTrue(kubernetes.kubernetes.client.CoreV1Api().
                                list_namespaced_pod().to_dict.called)

    def test_delete_deployments(self):
        '''
        Tests deployment deletion
        :return:
        '''
        with mock_kubernetes_library() as mock_kubernetes_lib:
            with patch('salt.modules.kubernetes.show_deployment', Mock(return_value=None)):
                with patch.dict(kubernetes.__salt__, {'config.option': Mock(return_value="")}):
                    mock_kubernetes_lib.client.V1DeleteOptions = Mock(return_value="")
                    mock_kubernetes_lib.client.ExtensionsV1beta1Api.return_value = Mock(
                        **{"delete_namespaced_deployment.return_value.to_dict.return_value": {'code': ''}}
                    )
                    self.assertEqual(kubernetes.delete_deployment("test"), {'code': 200})
                    self.assertTrue(
                        kubernetes.kubernetes.client.ExtensionsV1beta1Api().
                        delete_namespaced_deployment().to_dict.called)

    def test_create_deployments(self):
        '''
        Tests deployment creation.
        :return:
        '''
        with mock_kubernetes_library() as mock_kubernetes_lib:
            with patch.dict(kubernetes.__salt__, {'config.option': Mock(return_value="")}):
                mock_kubernetes_lib.client.ExtensionsV1beta1Api.return_value = Mock(
                    **{"create_namespaced_deployment.return_value.to_dict.return_value": {}}
                )
                self.assertEqual(kubernetes.create_deployment("test", "default", {}, {},
                                                              None, None, None), {})
                self.assertTrue(
                    kubernetes.kubernetes.client.ExtensionsV1beta1Api().
                    create_namespaced_deployment().to_dict.called)

    def test_setup_client_key_file(self):
        '''
        Test that the `kubernetes.client-key-file` configuration isn't overwritten
        :return:
        '''
        def settings(name, value=None):
            data = {
                'kubernetes.client-key-file': '/home/testuser/.minikube/client.key',
            }
            return data.get(name, value)

        with patch.dict(kubernetes.__salt__, {'config.option': Mock(side_effect=settings)}):
            config = kubernetes._setup_conn()
            self.assertEqual(
                settings('kubernetes.client-key-file'),
                config['key_file'],
            )

    def test_node_labels(self):
        '''
        Test kubernetes.node_labels
        :return:
        '''
        with patch('salt.modules.kubernetes.node') as mock_node:
            mock_node.return_value = {
                'metadata': {
                    'labels': {
                        'kubernetes.io/hostname': 'minikube',
                        'kubernetes.io/os': 'linux',
                    }
                }
            }
            self.assertEqual(
                kubernetes.node_labels('minikube'),
                {'kubernetes.io/hostname': 'minikube', 'kubernetes.io/os': 'linux'},
            )

    def test_adding_change_cause_annotation(self):
        '''
        Tests adding a `kubernetes.io/change-cause` annotation just like
        kubectl [apply|create|replace] --record does
        :return:
        '''
        with patch('salt.modules.kubernetes.sys.argv', ['/usr/bin/salt-call', 'state.apply']) as mock_sys:
            func = getattr(kubernetes, '__dict_to_object_meta')
            data = func(name='test-pod', namespace='test', metadata={})

            self.assertEqual(data.name, 'test-pod')
            self.assertEqual(data.namespace, 'test')
            self.assertEqual(
                data.annotations,
                {'kubernetes.io/change-cause': '/usr/bin/salt-call state.apply'}
            )

            # Ensure any specified annotations aren't overwritten
            test_metadata = {'annotations': {'kubernetes.io/change-cause': 'NOPE'}}
            data = func(name='test-pod', namespace='test', metadata=test_metadata)

            self.assertEqual(
                data.annotations,
                {'kubernetes.io/change-cause': 'NOPE'}
            )

    def test_enforce_only_strings_dict(self):
        func = getattr(kubernetes, '__enforce_only_strings_dict')
        data = {
            'unicode': 1,
            2: 2,
        }
        self.assertEqual(
            {'unicode': '1', '2': '2'},
            func(data),
        )



# -*- coding: utf-8 -*-
'''
Tests for salt.utils.iam
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import salt.utils.iam as iam
from salt.ext import six
from tests.support.unit import TestCase, skipIf


class IamTestCase(TestCase):

    @skipIf(six.PY3, 'Only needs to run on Python 2')
    def test__convert_key_to_str(self):
        '''
        Makes sure that unicode string is converted to a str type on PY2
        '''
        key = 'foo'
        expected = key.encode('utf-8')
        self.assertEqual(iam._convert_key_to_str(key), expected)



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import copy

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.paths import TMP
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt libs
from salt.exceptions import SaltException
import salt.modules.grains as grainsmod
import salt.utils.dictupdate as dictupdate

# Import 3rd-party libs
from salt.utils.odict import OrderedDict


@skipIf(NO_MOCK, NO_MOCK_REASON)
class GrainsModuleTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        conf_file = os.path.join(TMP, '__salt_test_grains')
        cachedir = os.path.join(TMP, '__salt_test_grains_cache_dir')
        if not os.path.isdir(cachedir):
            os.makedirs(cachedir)
        return {
            grainsmod: {
                '__opts__': {
                  'conf_file': conf_file,
                  'cachedir':  cachedir
                },
                '__salt__': {'saltutil.refresh_grains': MagicMock()}
            }
        }

    def test_filter_by(self):
        with patch.dict(grainsmod.__grains__, {'os_family': 'MockedOS',
                                               '1': '1',
                                               '2': '2',
                                               'roles': ['A', 'B']}):

            dict1 = {'A': 'B', 'C': {'D': {'E': 'F', 'G': 'H'}}}
            dict2 = {
                'default': {
                    'A': 'B',
                    'C': {
                        'D': 'E'
                    },
                },
                '1': {
                    'A': 'X',
                },
                '2': {
                    'C': {
                        'D': 'H',
                    },
                },
                'MockedOS': {
                    'A': 'Z',
                },
            }

            mdict1 = {'D': {'E': 'I'}, 'J': 'K'}
            mdict2 = {'A': 'Z'}
            mdict3 = {'C': {'D': 'J'}}

            # test None result with non existent grain and no default
            res = grainsmod.filter_by(dict1, grain='xxx')
            self.assertIs(res, None)

            # test None result with os_family grain and no matching result
            res = grainsmod.filter_by(dict1)
            self.assertIs(res, None)

            # test with non existent grain, and a given default key
            res = grainsmod.filter_by(dict1, grain='xxx', default='C')
            self.assertEqual(res, {'D': {'E': 'F', 'G': 'H'}})

            # add a merge dictionary, F disappears
            res = grainsmod.filter_by(dict1, grain='xxx', merge=mdict1, default='C')
            self.assertEqual(res, {'D': {'E': 'I', 'G': 'H'}, 'J': 'K'})
            # dict1 was altered, reestablish
            dict1 = {'A': 'B', 'C': {'D': {'E': 'F', 'G': 'H'}}}

            # default is not present in dict1, check we only have merge in result
            res = grainsmod.filter_by(dict1, grain='xxx', merge=mdict1, default='Z')
            self.assertEqual(res, mdict1)

            # default is not present in dict1, and no merge, should get None
            res = grainsmod.filter_by(dict1, grain='xxx', default='Z')
            self.assertIs(res, None)

            #test giving a list as merge argument raise exception
            self.assertRaises(
                SaltException,
                grainsmod.filter_by,
                dict1,
                'xxx',
                ['foo'],
                'C'
            )

            #Now, re-test with an existing grain (os_family), but with no match.
            res = grainsmod.filter_by(dict1)
            self.assertIs(res, None)
            res = grainsmod.filter_by(dict1, default='C')
            self.assertEqual(res, {'D': {'E': 'F', 'G': 'H'}})
            res = grainsmod.filter_by(dict1, merge=mdict1, default='C')
            self.assertEqual(res, {'D': {'E': 'I', 'G': 'H'}, 'J': 'K'})
            # dict1 was altered, reestablish
            dict1 = {'A': 'B', 'C': {'D': {'E': 'F', 'G': 'H'}}}
            res = grainsmod.filter_by(dict1, merge=mdict1, default='Z')
            self.assertEqual(res, mdict1)
            res = grainsmod.filter_by(dict1, default='Z')
            self.assertIs(res, None)
            # this one is in fact a traceback in updatedict, merging a string with a dictionary
            self.assertRaises(
                TypeError,
                grainsmod.filter_by,
                dict1,
                merge=mdict1,
                default='A'
            )

            #Now, re-test with a matching grain.
            dict1 = {'A': 'B', 'MockedOS': {'D': {'E': 'F', 'G': 'H'}}}
            res = grainsmod.filter_by(dict1)
            self.assertEqual(res, {'D': {'E': 'F', 'G': 'H'}})
            res = grainsmod.filter_by(dict1, default='A')
            self.assertEqual(res, {'D': {'E': 'F', 'G': 'H'}})
            res = grainsmod.filter_by(dict1, merge=mdict1, default='A')
            self.assertEqual(res, {'D': {'E': 'I', 'G': 'H'}, 'J': 'K'})
            # dict1 was altered, reestablish
            dict1 = {'A': 'B', 'MockedOS': {'D': {'E': 'F', 'G': 'H'}}}
            res = grainsmod.filter_by(dict1, merge=mdict1, default='Z')
            self.assertEqual(res, {'D': {'E': 'I', 'G': 'H'}, 'J': 'K'})
            # dict1 was altered, reestablish
            dict1 = {'A': 'B', 'MockedOS': {'D': {'E': 'F', 'G': 'H'}}}
            res = grainsmod.filter_by(dict1, default='Z')
            self.assertEqual(res, {'D': {'E': 'F', 'G': 'H'}})

            # Test when grain value is a list
            dict1 = {'A': 'B', 'C': {'D': {'E': 'F', 'G': 'H'}}}
            res = grainsmod.filter_by(dict1, grain='roles', default='C')
            self.assertEqual(res, 'B')
            # Test default when grain value is a list
            dict1 = {'Z': 'B', 'C': {'D': {'E': 'F', 'G': 'H'}}}
            res = grainsmod.filter_by(dict1, grain='roles', default='C')
            self.assertEqual(res, {'D': {'E': 'F', 'G': 'H'}})

            # Test with wildcard pattern in the lookup_dict keys
            dict1 = {'*OS': 'B', 'C': {'D': {'E': 'F', 'G': 'H'}}}
            res = grainsmod.filter_by(dict1)
            self.assertEqual(res, 'B')
            # Test with non-strings in lookup_dict keys
            # Issue #38094
            dict1 = {1: 2, 3: {4: 5}, '*OS': 'B'}
            res = grainsmod.filter_by(dict1)
            self.assertEqual(res, 'B')
            # Test with sequence pattern with roles
            dict1 = {'Z': 'B', '[BC]': {'D': {'E': 'F', 'G': 'H'}}}
            res = grainsmod.filter_by(dict1, grain='roles', default='Z')
            self.assertEqual(res, {'D': {'E': 'F', 'G': 'H'}})

            # Base tests
            # NOTE: these may fail to detect errors if dictupdate.update() is broken
            # but then the unit test for dictupdate.update() should fail and expose
            # that.  The purpose of these tests is it validate the logic of how
            # in filter_by() processes its arguments.

            # Test with just the base
            res = grainsmod.filter_by(dict2, grain='xxx', default='xxx', base='default')
            self.assertEqual(res, dict2['default'])

            # Test the base with the OS grain look-up
            res = grainsmod.filter_by(dict2, default='xxx', base='default')
            self.assertEqual(
                res,
                dictupdate.update(copy.deepcopy(dict2['default']), dict2['MockedOS'])
            )

            # Test the base with default
            res = grainsmod.filter_by(dict2, grain='xxx', base='default')
            self.assertEqual(res, dict2['default'])

            res = grainsmod.filter_by(dict2, grain='1', base='default')
            self.assertEqual(
                res,
                dictupdate.update(copy.deepcopy(dict2['default']), dict2['1'])
            )

            res = grainsmod.filter_by(dict2, base='default', merge=mdict2)
            self.assertEqual(
                res,
                dictupdate.update(
                    dictupdate.update(
                        copy.deepcopy(dict2['default']),
                        dict2['MockedOS']),
                    mdict2
                )
            )

            res = grainsmod.filter_by(dict2, base='default', merge=mdict3)
            self.assertEqual(
                res,
                dictupdate.update(
                    dictupdate.update(
                        copy.deepcopy(dict2['default']),
                        dict2['MockedOS']),
                    mdict3
                )
            )

    def test_append_not_a_list(self):
        # Failing append to an existing string, without convert
        with patch.dict(grainsmod.__grains__, {'b': 'bval'}):
            res = grainsmod.append('b', 'd')
            self.assertEqual(res, 'The key b is not a valid list')
            self.assertEqual(grainsmod.__grains__, {'b': 'bval'})

            # Failing append to an existing dict
            with patch.dict(grainsmod.__grains__, {'b': {'b1': 'bval1'}}):
                res = grainsmod.append('b', 'd')
                self.assertEqual(res, 'The key b is not a valid list')
                self.assertEqual(grainsmod.__grains__, {'b': {'b1': 'bval1'}})

    def test_append_already_in_list(self):
        # Append an existing value
        with patch.dict(grainsmod.__grains__, {'a_list': ['a', 'b', 'c'], 'b': 'bval'}):
            res = grainsmod.append('a_list', 'b')
            self.assertEqual(res, 'The val b was already in the list a_list')
            self.assertEqual(grainsmod.__grains__, {'a_list': ['a', 'b', 'c'], 'b': 'bval'})

    def test_append_ok(self):
        # Append to an existing list
        with patch.dict(grainsmod.__grains__, {'a_list': ['a', 'b', 'c'], 'b': 'bval'}):
            res = grainsmod.append('a_list', 'd')
            self.assertEqual(res, {'a_list': ['a', 'b', 'c', 'd']})
            self.assertEqual(grainsmod.__grains__, {'a_list': ['a', 'b', 'c', 'd'], 'b': 'bval'})

        # Append to an non existing list
        with patch.dict(grainsmod.__grains__, {'b': 'bval'}):
            res = grainsmod.append('a_list', 'd')
            self.assertEqual(res, {'a_list': ['d']})
            self.assertEqual(grainsmod.__grains__, {'a_list': ['d'], 'b': 'bval'})

        # Append to an existing string, with convert
        with patch.dict(grainsmod.__grains__, {'b': 'bval'}):
            res = grainsmod.append('b', 'd', convert=True)
            self.assertEqual(res, {'b': ['bval', 'd']})
            self.assertEqual(grainsmod.__grains__, {'b': ['bval', 'd']})

        # Append to an existing dict, with convert
        with patch.dict(grainsmod.__grains__, {'b': {'b1': 'bval1'}}):
            res = grainsmod.append('b', 'd', convert=True)
            self.assertEqual(res, {'b': [{'b1': 'bval1'}, 'd']})
            self.assertEqual(grainsmod.__grains__, {'b': [{'b1': 'bval1'}, 'd']})

    def test_append_nested_not_a_list(self):
        # Failing append to an existing string, without convert
        with patch.dict(grainsmod.__grains__, {'a': {'b': 'bval'}}):
            res = grainsmod.append('a:b', 'd')
            self.assertEqual(res, 'The key a:b is not a valid list')
            self.assertEqual(grainsmod.__grains__, {'a': {'b': 'bval'}})

        # Failing append to an existing dict
        with patch.dict(grainsmod.__grains__, {'a': {'b': {'b1': 'bval1'}}}):
            res = grainsmod.append('a:b', 'd')
            self.assertEqual(res, 'The key a:b is not a valid list')
            self.assertEqual(grainsmod.__grains__, {'a': {'b': {'b1': 'bval1'}}})

    def test_append_nested_already_in_list(self):
        # Append an existing value
        with patch.dict(grainsmod.__grains__, {'a': {'a_list': ['a', 'b', 'c'], 'b': 'bval'}}):
            res = grainsmod.append('a:a_list', 'b')
            self.assertEqual(res, 'The val b was already in the list a:a_list')
            self.assertEqual(grainsmod.__grains__, {'a': {'a_list': ['a', 'b', 'c'], 'b': 'bval'}})

    def test_append_nested_ok(self):
        # Append to an existing list
        with patch.dict(grainsmod.__grains__, {'a': {'a_list': ['a', 'b', 'c'], 'b': 'bval'}}):
            res = grainsmod.append('a:a_list', 'd')
            self.assertEqual(res, {'a': {'a_list': ['a', 'b', 'c', 'd'], 'b': 'bval'}})
            self.assertEqual(grainsmod.__grains__, {'a': {'a_list': ['a', 'b', 'c', 'd'], 'b': 'bval'}})

        # Append to an non existing list
        with patch.dict(grainsmod.__grains__, {'a': {'b': 'bval'}}):
            res = grainsmod.append('a:a_list', 'd')
            self.assertEqual(res, {'a': {'a_list': ['d'], 'b': 'bval'}})
            self.assertEqual(grainsmod.__grains__, {'a': {'a_list': ['d'], 'b': 'bval'}})

        # Append to an existing string, with convert
        with patch.dict(grainsmod.__grains__, {'a': {'b': 'bval'}}):
            res = grainsmod.append('a:b', 'd', convert=True)
            self.assertEqual(res, {'a': {'b': ['bval', 'd']}})
            self.assertEqual(grainsmod.__grains__, {'a': {'b': ['bval', 'd']}})

        # Append to an existing dict, with convert
        with patch.dict(grainsmod.__grains__, {'a': {'b': {'b1': 'bval1'}}}):
            res = grainsmod.append('a:b', 'd', convert=True)
            self.assertEqual(res, {'a': {'b': [{'b1': 'bval1'}, 'd']}})
            self.assertEqual(grainsmod.__grains__, {'a': {'b': [{'b1': 'bval1'}, 'd']}})

    def test_append_to_an_element_of_a_list(self):
        # Append to an element in a list
        # It currently fails silently
        with patch.dict(grainsmod.__grains__, {'a': ['b', 'c']}):
            res = grainsmod.append('a:b', 'd')
            self.assertEqual(res, {'a': ['b', 'c']})
            self.assertEqual(grainsmod.__grains__, {'a': ['b', 'c']})

    def test_set_value_already_set(self):
        # Set a grain to the same simple value
        with patch.dict(grainsmod.__grains__, {'a': 12, 'c': 8}):
            res = grainsmod.set('a', 12)
            self.assertTrue(res['result'])
            self.assertEqual(res['comment'], 'Grain is already set')
            self.assertEqual(grainsmod.__grains__, {'a': 12, 'c': 8})

        # Set a grain to the same complex value
        with patch.dict(grainsmod.__grains__, {'a': ['item', 12], 'c': 8}):
            res = grainsmod.set('a', ['item', 12])
            self.assertTrue(res['result'])
            self.assertEqual(res['comment'], 'Grain is already set')
            self.assertEqual(grainsmod.__grains__, {'a': ['item', 12], 'c': 8})

        # Set a key to the same simple value in a nested grain
        with patch.dict(grainsmod.__grains__, {'a': 'aval', 'b': {'nested': 'val'}, 'c': 8}):
            res = grainsmod.set('b,nested', 'val', delimiter=',')
            self.assertTrue(res['result'])
            self.assertEqual(res['comment'], 'Grain is already set')
            self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                    'b': {'nested': 'val'},
                                                    'c': 8})

    def test_set_fail_replacing_existing_complex_key(self):
        # Fails to set a complex value without 'force'
        with patch.dict(grainsmod.__grains__, {'a': 'aval', 'c': 8}):
            res = grainsmod.set('a', ['item', 12])
            self.assertFalse(res['result'])
            self.assertEqual(res['comment'], 'The key \'a\' exists and the given value is a '
                                             'dict or a list. Use \'force=True\' to overwrite.')
            self.assertEqual(grainsmod.__grains__, {'a': 'aval', 'c': 8})

        # Fails to overwrite a complex value without 'force'
        with patch.dict(grainsmod.__grains__, {'a': ['item', 12], 'c': 8}):
            res = grainsmod.set('a', ['item', 14])
            self.assertFalse(res['result'])
            self.assertEqual(res['comment'], 'The key \'a\' exists but is a dict or a list. '
                                + 'Use \'force=True\' to overwrite.')
            self.assertEqual(grainsmod.__grains__, {'a': ['item', 12], 'c': 8})

        # Fails to overwrite a complex value without 'force' in a nested grain
        with patch.dict(grainsmod.__grains__, {'a': 'aval',
                                               'b': ['l1', {'l2': ['val1']}],
                                               'c': 8}):
            res = grainsmod.set('b,l2', 'val2', delimiter=',')
            self.assertFalse(res['result'])
            self.assertEqual(res['comment'], 'The key \'b,l2\' exists but is a dict or a '
                                + 'list. Use \'force=True\' to overwrite.')
            self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                    'b': ['l1', {'l2': ['val1']}],
                                                    'c': 8})

    def test_set_nested_fails_replace_simple_value(self):
        # Fails to replace a simple value with a new dictionary consisting
        # of the specified key and value
        with patch.dict(grainsmod.__grains__, {'a': 'aval', 'b': 'l1', 'c': 8}):
            res = grainsmod.set('b,l3', 'val3', delimiter=',')
            self.assertFalse(res['result'])
            self.assertEqual(res['comment'], 'The key \'b\' value is \'l1\', which is '
                                + 'different from the provided key \'l3\'. '
                                + 'Use \'force=True\' to overwrite.')
            self.assertEqual(grainsmod.__grains__, {'a': 'aval', 'b': 'l1', 'c': 8})

    def test_set_simple_value(self):
        with patch.dict(grainsmod.__grains__, {'a': ['b', 'c'], 'c': 8}):
            res = grainsmod.set('b', 'bval')
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'b': 'bval'})
            self.assertEqual(grainsmod.__grains__, {'a': ['b', 'c'],
                                                    'b': 'bval',
                                                    'c': 8})

    def test_set_replace_value(self):
        with patch.dict(grainsmod.__grains__, {'a': 'aval', 'c': 8}):
            res = grainsmod.set('a', 12)
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'a': 12})
            self.assertEqual(grainsmod.__grains__, {'a': 12, 'c': 8})

    def test_set_None_ok(self):
        with patch.dict(grainsmod.__grains__, {'a': 'aval', 'c': 8}):
            res = grainsmod.set('b', None)
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'b': None})
            self.assertEqual(grainsmod.__grains__, {'a': 'aval', 'b': None, 'c': 8})

    def test_set_None_ok_destructive(self):
        with patch.dict(grainsmod.__grains__, {'a': 'aval', 'c': 8}):
            res = grainsmod.set('b', None, destructive=True)
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'b': None})
            self.assertEqual(grainsmod.__grains__, {'a': 'aval', 'c': 8})

    def test_set_None_replace_ok(self):
        with patch.dict(grainsmod.__grains__, {'a': 'aval', 'c': 8}):
            res = grainsmod.set('a', None)
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'a': None})
            self.assertEqual(grainsmod.__grains__, {'a': None, 'c': 8})

    def test_set_None_force_destructive(self):
        with patch.dict(grainsmod.__grains__, {'a': 'aval', 'c': 8}):
            res = grainsmod.set('a', None, force=True, destructive=True)
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'a': None})
            self.assertEqual(grainsmod.__grains__, {'c': 8})

    def test_set_replace_value_was_complex_force(self):
        with patch.dict(grainsmod.__grains__, {'a': ['item', 12], 'c': 8}):
            res = grainsmod.set('a', 'aval', force=True)
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'a': 'aval'})
            self.assertEqual(grainsmod.__grains__, {'a': 'aval', 'c': 8})

    def test_set_complex_value_force(self):
        with patch.dict(grainsmod.__grains__, {'a': 'aval', 'c': 8}):
            res = grainsmod.set('a', ['item', 12], force=True)
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'a': ['item', 12]})
            self.assertEqual(grainsmod.__grains__, {'a': ['item', 12], 'c': 8})

    def test_set_nested_create(self):
        with patch.dict(grainsmod.__grains__, {'a': 'aval', 'c': 8}):
            res = grainsmod.set('b,nested', 'val', delimiter=',')
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'b': {'nested': 'val'}})
            self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                    'b': {'nested': 'val'},
                                                    'c': 8})

    def test_set_nested_update_dict(self):
        with patch.dict(grainsmod.__grains__, {'a': 'aval', 'b': {'nested': 'val'}, 'c': 8}):
            res = grainsmod.set('b,nested', 'val2', delimiter=',')
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'b': {'nested': 'val2'}})
            self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                    'b': {'nested': 'val2'},
                                                    'c': 8})

    def test_set_nested_update_dict_remove_key(self):
        with patch.dict(grainsmod.__grains__, {'a': 'aval', 'b': {'nested': 'val'}, 'c': 8}):
            res = grainsmod.set('b,nested', None, delimiter=',', destructive=True)
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'b': {}})
            self.assertEqual(grainsmod.__grains__, {'a': 'aval', 'b': {}, 'c': 8})

    def test_set_nested_update_dict_new_key(self):
        with patch.dict(grainsmod.__grains__, {'a': 'aval', 'b': {'nested': 'val'}, 'c': 8}):
            res = grainsmod.set('b,b2', 'val2', delimiter=',')
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'b': {'b2': 'val2', 'nested': 'val'}})
            self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                    'b': {'b2': 'val2',
                                                          'nested': 'val'},
                                                    'c': 8})

    def test_set_nested_list_replace_key(self):
        with patch.dict(grainsmod.__grains__, {'a': 'aval', 'b': ['l1', 'l2', 'l3'], 'c': 8}):
            res = grainsmod.set('b,l2', 'val2', delimiter=',')
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'b': ['l1', {'l2': 'val2'}, 'l3']})
            self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                    'b': ['l1', {'l2': 'val2'}, 'l3'],
                                                    'c': 8})

    def test_set_nested_list_update_dict_key(self):
        with patch.dict(grainsmod.__grains__, {'a': 'aval', 'b': ['l1', {'l2': 'val1'}], 'c': 8}):
            res = grainsmod.set('b,l2', 'val2', delimiter=',')
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'b': ['l1', {'l2': 'val2'}]})
            self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                    'b': ['l1', {'l2': 'val2'}],
                                                    'c': 8})

    def test_set_nested_list_update_dict_key_overwrite(self):
        with patch.dict(grainsmod.__grains__, {'a': 'aval',
                                               'b': ['l1', {'l2': ['val1']}],
                                               'c': 8}):
            res = grainsmod.set('b,l2', 'val2', delimiter=',', force=True)
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'b': ['l1', {'l2': 'val2'}]})
            self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                    'b': ['l1', {'l2': 'val2'}],
                                                    'c': 8})

    def test_set_nested_list_append_dict_key(self):
        with patch.dict(grainsmod.__grains__, {'a': 'aval',
                                               'b': ['l1', {'l2': 'val2'}],
                                               'c': 8}):
            res = grainsmod.set('b,l3', 'val3', delimiter=',')
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'b': ['l1', {'l2': 'val2'}, {'l3': 'val3'}]})
            self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                    'b': ['l1',
                                                          {'l2': 'val2'},
                                                          {'l3': 'val3'}],
                                                    'c': 8})

    def test_set_nested_existing_value_is_the_key(self):
        with patch.dict(grainsmod.__grains__, {'a': 'aval', 'b': 'l3', 'c': 8}):
            res = grainsmod.set('b,l3', 'val3', delimiter=',')
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'b': {'l3': 'val3'}})
            self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                    'b': {'l3': 'val3'},
                                                    'c': 8})

    def test_set_nested_existing_value_overwrite(self):
        with patch.dict(grainsmod.__grains__, {'a': 'aval', 'b': 'l1', 'c': 8}):
            res = grainsmod.set('b,l3', 'val3', delimiter=',', force=True)
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'b': {'l3': 'val3'}})
            self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                    'b': {'l3': 'val3'},
                                                    'c': 8})

    def test_set_deeply_nested_update(self):
        with patch.dict(grainsmod.__grains__, {'a': 'aval',
                                               'b': {'l1': ['l21', 'l22', {'l23': 'l23val'}]},
                                               'c': 8}):
            res = grainsmod.set('b,l1,l23', 'val', delimiter=',')
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'b': {'l1': ['l21', 'l22', {'l23': 'val'}]}})
            self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                    'b': {'l1': ['l21',
                                                                 'l22',
                                                                 {'l23': 'val'}]},
                                                    'c': 8})

    def test_set_deeply_nested_create(self):
        with patch.dict(grainsmod.__grains__, {'a': 'aval',
                                               'b': {'l1': ['l21', 'l22', {'l23': 'l23val'}]},
                                               'c': 8}):
            res = grainsmod.set('b,l1,l24,l241', 'val', delimiter=',')
            self.assertTrue(res['result'])
            self.assertEqual(res['changes'], {'b': {'l1': ['l21',
                                                'l22',
                                                {'l23': 'l23val'},
                                                {'l24': {'l241': 'val'}}]}})
            self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                    'b': {'l1': [
                                                        'l21',
                                                        'l22',
                                                        {'l23': 'l23val'},
                                                        {'l24': {'l241': 'val'}}]},
                                                    'c': 8})

    def test_get_ordered(self):
        with patch.dict(grainsmod.__grains__, OrderedDict([
                                                ('a', 'aval'),
                                                ('b', OrderedDict([
                                                    ('z', 'zval'),
                                                    ('l1', ['l21',
                                                            'l22',
                                                            OrderedDict([('l23', 'l23val')])])
                                                    ])),
                                                ('c', 8)])):
            res = grainsmod.get('b')
            self.assertEqual(type(res), OrderedDict)
            # Check that order really matters
            self.assertTrue(res == OrderedDict([
                                      ('z', 'zval'),
                                      ('l1', ['l21',
                                              'l22',
                                              OrderedDict([('l23', 'l23val')])]),
                                      ]))
            self.assertFalse(res == OrderedDict([
                                      ('l1', ['l21',
                                              'l22',
                                              OrderedDict([('l23', 'l23val')])]),
                                      ('z', 'zval'),
                                      ]))

    def test_get_unordered(self):
        with patch.dict(grainsmod.__grains__, OrderedDict([
                                                ('a', 'aval'),
                                                ('b', OrderedDict([
                                                    ('z', 'zval'),
                                                    ('l1', ['l21',
                                                            'l22',
                                                            OrderedDict([('l23', 'l23val')])])
                                                    ])),
                                                ('c', 8)])):
            res = grainsmod.get('b', ordered=False)
            self.assertEqual(type(res), dict)
            # Check that order doesn't matter
            self.assertTrue(res == OrderedDict([
                                      ('l1', ['l21',
                                              'l22',
                                              OrderedDict([('l23', 'l23val')])]),
                                      ('z', 'zval'),
                                      ]))

    def test_equals(self):
        with patch.dict(grainsmod.__grains__, OrderedDict([
                                                ('a', 'aval'),
                                                ('b', OrderedDict([
                                                    ('z', 'zval'),
                                                    ('l1', ['l21',
                                                            'l22',
                                                            OrderedDict([('l23', 'l23val')])])
                                                    ])),
                                                ('c', 8)])):
            res = grainsmod.equals('a', 'aval')
            self.assertEqual(type(res), bool)
            self.assertTrue(res)
            res = grainsmod.equals('b:z', 'zval')
            self.assertTrue(res)
            res = grainsmod.equals('b:z', 'aval')
            self.assertFalse(res)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.saltcloudmod as saltcloudmod
import salt.utils.json


@skipIf(NO_MOCK, NO_MOCK_REASON)
class SaltcloudmodTestCase(TestCase, LoaderModuleMockMixin):
    '''
        Test cases for salt.modules.saltcloudmod
    '''
    def setup_loader_modules(self):
        return {saltcloudmod: {}}

    def setUp(self):
        self.mock_json_loads = MagicMock(side_effect=ValueError())

    def test_create(self):
        '''
            Test if create the named vm
        '''
        mock = MagicMock(return_value='''{"foo": "bar"}''')
        with patch.dict(saltcloudmod.__salt__, {'cmd.run_stdout': mock}):
            self.assertTrue(
                saltcloudmod.create("webserver", "rackspace_centos_512"))

            with patch.object(salt.utils.json, 'loads', self.mock_json_loads):
                self.assertDictEqual(
                    saltcloudmod.create("webserver", "rackspace_centos_512"),
                    {}
                )



# -*- coding: utf-8 -*-
'''
    :codeauthor: Alexander Schwartz <alexander.schwartz@gmx.net>
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import MagicMock, NO_MOCK, NO_MOCK_REASON, patch, ANY


# Import Salt Libs
import salt.client
from salt.cloud.clouds import saltify

TEST_PROFILES = {
    'testprofile1': NotImplemented,
    'testprofile2': {  # this profile is used in test_saltify_destroy()
                     'ssh_username': 'fred',
                     'remove_config_on_destroy': False,  # expected for test
                     'shutdown_on_destroy': True  # expected value for test
                     },
    'testprofile3': {  # this profile is used in test_create_wake_on_lan()
                    'wake_on_lan_mac': 'aa-bb-cc-dd-ee-ff',
                    'wol_sender_node': 'friend1',
                    'wol_boot_wait': 0.01  # we want the wait to be very short
                    }
    }
TEST_PROFILE_NAMES = ['testprofile1', 'testprofile2', 'testprofile3']


@skipIf(NO_MOCK, NO_MOCK_REASON)
class SaltifyTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.cloud.clouds.saltify
    '''
    LOCAL_OPTS = {
        'providers': {
            'sfy1': {
                'saltify': {
                    'driver': 'saltify',
                    'profiles': TEST_PROFILES
                    }
                },
            },
        'profiles': TEST_PROFILES,
        'sock_dir': '/var/sockxxx',
        'transport': 'tcp',
        }

    def setup_loader_modules(self):
        saltify_globals = {
                '__active_provider_name__': '',
                '__utils__': {
                    'cloud.bootstrap': MagicMock(),
                    'cloud.fire_event': MagicMock(),
                    },
                '__opts__': self.LOCAL_OPTS,
                }
        return {saltify: saltify_globals}

    def test_create_no_deploy(self):
        '''
        Test if deployment fails. This is the most basic test as saltify doesn't contain much logic
        '''
        with patch('salt.cloud.clouds.saltify._verify', MagicMock(return_value=True)):
            vm = {'deploy':  False,
                  'driver': 'saltify',
                  'name': 'dummy'
                  }
            self.assertTrue(saltify.create(vm))

    def test_create_and_deploy(self):
        '''
        Test if deployment can be done.
        '''
        mock_cmd = MagicMock(return_value=True)
        with patch.dict(
            'salt.cloud.clouds.saltify.__utils__',
            {'cloud.bootstrap': mock_cmd}):
            vm_ = {'deploy':  True,
                  'driver': 'saltify',
                  'name': 'new2',
                  'profile': 'testprofile2',
                 }
            result = saltify.create(vm_)
            mock_cmd.assert_called_once_with(vm_, ANY)
            self.assertTrue(result)

    def test_create_wake_on_lan(self):
        '''
        Test if wake on lan works
        '''
        mock_sleep = MagicMock()
        mock_cmd = MagicMock(return_value=True)
        mm_cmd = MagicMock(return_value={'friend1': True})
        lcl = salt.client.LocalClient()
        lcl.cmd = mm_cmd
        with patch('time.sleep', mock_sleep):
            with patch('salt.client.LocalClient', return_value=lcl):
                with patch.dict(
                    'salt.cloud.clouds.saltify.__utils__',
                    {'cloud.bootstrap': mock_cmd}):
                    vm_ = {'deploy': True,
                           'driver': 'saltify',
                           'name': 'new1',
                           'profile': 'testprofile3',
                           }
                    result = saltify.create(vm_)
                    mock_cmd.assert_called_once_with(vm_, ANY)
                    mm_cmd.assert_called_with('friend1', 'network.wol', ['aa-bb-cc-dd-ee-ff'])
                    mock_sleep.assert_called_with(0.01)
                    self.assertTrue(result)

    def test_avail_locations(self):
        '''
        Test the avail_locations will always return {}
        '''
        self.assertEqual(saltify.avail_locations(), {})

    def test_avail_sizes(self):
        '''
        Test the avail_sizes will always return {}
        '''
        self.assertEqual(saltify.avail_sizes(), {})

    def test_avail_images(self):
        '''
        Test the avail_images will return profiles
        '''
        testlist = list(TEST_PROFILE_NAMES)  # copy
        self.assertEqual(
            saltify.avail_images()['Profiles'].sort(),
            testlist.sort())

    def test_list_nodes(self):
        '''
        Test list_nodes will return required fields only
        '''
        testgrains = {
            'nodeX1': {
                'id': 'nodeX1',
                'ipv4': [
                    '127.0.0.1', '192.1.2.22', '172.16.17.18'],
                'ipv6': [
                    '::1', 'fdef:bad:add::f00', '3001:DB8::F00D'],
                'salt-cloud': {
                    'driver': 'saltify',
                    'provider': 'saltyfy',
                    'profile': 'testprofile2'
                   },
                'extra_stuff': 'does not belong'
                }
            }
        expected_result = {
            'nodeX1': {
                'id': 'nodeX1',
                'image': 'testprofile2',
                'private_ips': [
                    '172.16.17.18', 'fdef:bad:add::f00'],
                'public_ips': [
                    '192.1.2.22', '3001:DB8::F00D'],
                'size': '',
                'state': 'running'
                }
            }
        mm_cmd = MagicMock(return_value=testgrains)
        lcl = salt.client.LocalClient()
        lcl.cmd = mm_cmd
        with patch('salt.client.LocalClient', return_value=lcl):
            self.assertEqual(
                saltify.list_nodes(),
                expected_result)

    def test_saltify_reboot(self):
        mm_cmd = MagicMock(return_value=True)
        lcl = salt.client.LocalClient()
        lcl.cmd = mm_cmd
        with patch('salt.client.LocalClient', return_value=lcl):
            result = saltify.reboot('nodeS1', 'action')
            mm_cmd.assert_called_with('nodeS1', 'system.reboot')
            self.assertTrue(result)

    def test_saltify_destroy(self):
        # destroy calls local.cmd several times and expects
        # different results, so we will provide a list of
        # results. Each call will get the next value.
        # NOTE: this assumes that the call order never changes,
        # so to keep things simple, we will not use remove_config...
        result_list = [
                {'nodeS1': {  # first call is grains.get
                    'driver': 'saltify',
                    'provider': 'saltify',
                    'profile': 'testprofile2'}
                },
                #  Note:
                #    testprofile2 has remove_config_on_destroy: False
                #    and shutdown_on_destroy: True
                {'nodeS1':  # last call shuts down the minion
                     'a system.shutdown worked message'},
            ]
        mm_cmd = MagicMock(side_effect=result_list)
        lcl = salt.client.LocalClient()
        lcl.cmd = mm_cmd
        with patch('salt.client.LocalClient', return_value=lcl):
            result = saltify.destroy('nodeS1', 'action')
            mm_cmd.assert_called_with('nodeS1', 'system.shutdown')
            self.assertTrue(result)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import tempfile

# Import Salt Libs
from salt.cloud.clouds import ec2
from salt.exceptions import SaltCloudSystemExit
import salt.utils.files

# Import Salt Testing Libs
from tests.support.unit import TestCase, skipIf
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch, PropertyMock
from tests.support.paths import TMP
from tests.unit.test_crypt import PRIVKEY_DATA

PASS_DATA = (
    b'qOjCKDlBdcNEbJ/J8eRl7sH+bYIIm4cvHHY86gh2NEUnufFlFo0gGVTZR05Fj0cw3n/w7gR'
    b'urNXz5JoeSIHVuNI3YTwzL9yEAaC0kuy8EbOlO2yx8yPGdfml9BRwOV7A6b8UFo9co4H7fz'
    b'DdScMKU2yzvRYvp6N6Q2cJGBmPsemnXWWusb+1vZVWxcRAQmG3ogF6Z5rZSYAYH0N4rqJgH'
    b'mQfzuyb+jrBvV/IOoV1EdO9jGSH9338aS47NjrmNEN/SpnS6eCWZUwwyHbPASuOvWiY4QH/'
    b'0YZC6EGccwiUmt0ZOxIynk+tEyVPTkiS0V8RcZK6YKqMWHpKmPtLBzfuoA=='
)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class EC2TestCase(TestCase, LoaderModuleMockMixin):
    '''
    Unit TestCase for salt.cloud.clouds.ec2 module.
    '''

    def setUp(self):
        super(EC2TestCase, self).setUp()
        with tempfile.NamedTemporaryFile(dir=TMP, suffix='.pem', delete=True) as fp:
            self.key_file = fp.name

    def tearDown(self):
        super(EC2TestCase, self).tearDown()
        if os.path.exists(self.key_file):
            os.remove(self.key_file)

    def setup_loader_modules(self):
        return {ec2: {'__opts__': {}}}

    def test__validate_key_path_and_mode(self):

        # Key file exists
        with patch('os.path.exists', return_value=True):
            with patch('os.stat') as patched_stat:

                type(patched_stat.return_value).st_mode = PropertyMock(return_value=0o644)
                self.assertRaises(
                    SaltCloudSystemExit, ec2._validate_key_path_and_mode, 'key_file')

                type(patched_stat.return_value).st_mode = PropertyMock(return_value=0o600)
                self.assertTrue(ec2._validate_key_path_and_mode('key_file'))

                type(patched_stat.return_value).st_mode = PropertyMock(return_value=0o400)
                self.assertTrue(ec2._validate_key_path_and_mode('key_file'))

        # Key file does not exist
        with patch('os.path.exists', return_value=False):
            self.assertRaises(
                SaltCloudSystemExit, ec2._validate_key_path_and_mode, 'key_file')

    @patch('salt.cloud.clouds.ec2._get_node')
    @patch('salt.cloud.clouds.ec2.get_location')
    @patch('salt.cloud.clouds.ec2.get_provider')
    @patch('salt.utils.aws.query')
    def test_get_password_data(self, query, get_provider, get_location, _get_node):
        query.return_value = [
            {
            'passwordData': PASS_DATA
            }
        ]
        _get_node.return_value = {'instanceId': 'i-abcdef'}
        get_location.return_value = 'us-west2'
        get_provider.return_value = 'ec2'
        with salt.utils.files.fopen(self.key_file, 'w') as fp:
            fp.write(PRIVKEY_DATA)
        ret = ec2.get_password_data(
            name='i-abcddef', kwargs={'key_file': self.key_file}, call='action'
        )
        assert ret['passwordData'] == PASS_DATA
        assert ret['password'] == b'testp4ss!'



# -*- coding: utf-8 -*-
'''
tests.unit.returners.local_cache_test
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Unit tests for the Default Job Cache (local_cache).
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import shutil
import time
import logging
import tempfile
import time

# Import Salt Testing libs
from tests.integration import AdaptedConfigurationTestCaseMixin
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.paths import TMP
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON,
    patch
)

# Import Salt libs
import salt.utils.files
import salt.utils.jid
import salt.utils.job
import salt.utils.platform
import salt.returners.local_cache as local_cache
from salt.ext import six

log = logging.getLogger(__name__)

TMP_CACHE_DIR = os.path.join(TMP, 'salt_test_job_cache')
TMP_JID_DIR = os.path.join(TMP_CACHE_DIR, 'jobs')


@skipIf(NO_MOCK, NO_MOCK_REASON)
class LocalCacheCleanOldJobsTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Tests for the local_cache.clean_old_jobs function.
    '''
    def setup_loader_modules(self):
        return {local_cache: {'__opts__': {'cachedir': TMP_CACHE_DIR, 'keep_jobs': 1}}}

    def tearDown(self):
        '''
        Clean up after tests.

        Note that a setUp function is not used in this TestCase because the
        _make_tmp_jid_dirs replaces it.
        '''
        if os.path.exists(TMP_CACHE_DIR):
            shutil.rmtree(TMP_CACHE_DIR)

    def test_clean_old_jobs_no_jid_root(self):
        '''
        Tests that the function returns None when no jid_root is found.
        '''
        with patch('os.path.exists', MagicMock(return_value=False)):
            self.assertEqual(local_cache.clean_old_jobs(), None)

    def test_clean_old_jobs_empty_jid_dir_removed(self):
        '''
        Tests that an empty JID dir is removed when it is old enough to be deleted.
        '''
        # Create temp job cache dir without files in it.
        jid_dir, jid_file = self._make_tmp_jid_dirs(create_files=False)

        # File timestamps on Windows aren't as precise. Let a little time pass
        if salt.utils.platform.is_windows():
            time.sleep(.01)

        # Make sure there are no files in the directory before continuing
        self.assertEqual(jid_file, None)

        # Call clean_old_jobs function, patching the keep_jobs value with a
        # very small value to force the call to clean the job.
        with patch.dict(local_cache.__opts__, {'keep_jobs': 0.00000001}):
            # Sleep on Windows because time.time is only precise to 3 decimal
            # points, and therefore subtracting the jid_ctime from time.time
            # will result in a negative number
            if salt.utils.platform.is_windows():
                time.sleep(0.25)
            local_cache.clean_old_jobs()

        # Assert that the JID dir was removed
        self.assertEqual([], os.listdir(TMP_JID_DIR))

    def test_clean_old_jobs_empty_jid_dir_remains(self):
        '''
        Tests that an empty JID dir is NOT removed because it was created within
        the keep_jobs time frame.
        '''
        # Create temp job cache dir without files in it.
        jid_dir, jid_file = self._make_tmp_jid_dirs(create_files=False)

        # Make sure there are no files in the directory
        self.assertEqual(jid_file, None)

        # Call clean_old_jobs function
        local_cache.clean_old_jobs()

        # Get the name of the JID directory that was created to test against
        if salt.utils.platform.is_windows():
            jid_dir_name = jid_dir.rpartition('\\')[2]
        else:
            jid_dir_name = jid_dir.rpartition('/')[2]

        # Assert the JID directory is still present to be cleaned after keep_jobs interval
        self.assertEqual([jid_dir_name], os.listdir(TMP_JID_DIR))

    def test_clean_old_jobs_jid_file_corrupted(self):
        '''
        Tests that the entire JID dir is removed when the jid_file is not a file.
        This scenario indicates a corrupted cache entry, so the entire dir is scrubbed.
        '''
        # Create temp job cache dir and jid file
        jid_dir, jid_file = self._make_tmp_jid_dirs()

        # Make sure there is a jid file in a new job cache director
        jid_dir_name = jid_file.rpartition('/')[2]
        self.assertEqual(jid_dir_name, 'jid')

        # Even though we created a valid jid file in the _make_tmp_jid_dirs call to get
        # into the correct loop, we need to mock the 'os.path.isfile' check to force the
        # "corrupted file" check in the clean_old_jobs call.
        with patch('os.path.isfile', MagicMock(return_value=False)) as mock:
            local_cache.clean_old_jobs()

        # Assert that the JID dir was removed
        self.assertEqual([], os.listdir(TMP_JID_DIR))

    def test_clean_old_jobs_jid_file_is_cleaned(self):
        '''
        Test that the entire JID dir is removed when a job is old enough to be removed.
        '''
        # Create temp job cache dir and jid file
        jid_dir, jid_file = self._make_tmp_jid_dirs()

        # File timestamps on Windows aren't as precise. Let a little time pass
        if salt.utils.platform.is_windows():
            time.sleep(.01)

        # Make sure there is a jid directory
        jid_dir_name = jid_file.rpartition('/')[2]
        self.assertEqual(jid_dir_name, 'jid')

        # Call clean_old_jobs function, patching the keep_jobs value with a
        # very small value to force the call to clean the job.
        with patch.dict(local_cache.__opts__, {'keep_jobs': 0.00000001}):
            # Sleep on Windows because time.time is only precise to 3 decimal
            # points, and therefore subtracting the jid_ctime from time.time
            # will result in a negative number
            if salt.utils.platform.is_windows():
                time.sleep(0.25)
            local_cache.clean_old_jobs()

        # Assert that the JID dir was removed
        self.assertEqual([], os.listdir(TMP_JID_DIR))

    def _make_tmp_jid_dirs(self, create_files=True):
        '''
        Helper function to set up temporary directories and files used for
        testing the clean_old_jobs function.

        Returns a temp_dir name and a jid_file_path. If create_files is False,
        the jid_file_path will be None.
        '''
        # First, create the /tmp/salt_test_job_cache/jobs/ directory to hold jid dirs
        if not os.path.exists(TMP_JID_DIR):
            os.makedirs(TMP_JID_DIR)

        # Then create a JID temp file in "/tmp/salt_test_job_cache/"
        temp_dir = tempfile.mkdtemp(dir=TMP_JID_DIR)

        jid_file_path = None
        if create_files:
            dir_name = '/'.join([temp_dir, 'jid'])
            os.mkdir(dir_name)
            jid_file_path = '/'.join([dir_name, 'jid'])
            with salt.utils.files.fopen(jid_file_path, 'w') as jid_file:
                jid_file.write('this is a jid file')

        return temp_dir, jid_file_path


class Local_CacheTest(TestCase, AdaptedConfigurationTestCaseMixin, LoaderModuleMockMixin):
    '''
    Test the local cache returner
    '''
    def setup_loader_modules(self):
        return {
            local_cache: {
                '__opts__': {
                    'cachedir': self.TMP_CACHE_DIR,
                    'keep_jobs': self.KEEP_JOBS
                }
            }
        }

    @classmethod
    def setUpClass(cls):
        cls.TMP_CACHE_DIR = os.path.join(TMP, 'rootdir', 'cache')
        cls.JOBS_DIR = os.path.join(cls.TMP_CACHE_DIR, 'jobs')
        cls.JID_DIR = os.path.join(cls.JOBS_DIR, '31', 'c56eed380a4e899ae12bc42563cfdfc53066fb4a6b53e2378a08ac49064539')
        cls.JID_FILE = os.path.join(cls.JID_DIR, 'jid')
        cls.JID_MINION_DIR = os.path.join(cls.JID_DIR, 'minion', 'return.p')
        cls.JOB_CACHE_DIR_FILES = [cls.JID_FILE, cls.JID_MINION_DIR]
        cls.KEEP_JOBS = 0.0000000010
        cls.EMPTY_JID_DIR = []

    @classmethod
    def tearDownClass(cls):
        for attrname in ('TMP_CACHE_DIR', 'JOBS_DIR', 'JID_DIR', 'JID_FILE', 'JID_MINION_DIR',
                         'JOB_CACHE_DIR_FILES', 'KEEP_JOBS', 'EMPTY_JID_DIR'):
            try:
                attr_instance = getattr(cls, attrname)
                if isinstance(attr_instance, six.string_types):
                    if os.path.isdir(attr_instance):
                        shutil.rmtree(attr_instance)
                    elif os.path.isfile(attr_instance):
                        os.unlink(attr_instance)
                delattr(cls, attrname)
            except AttributeError:
                continue

    def _check_dir_files(self, msg, contents, status='None'):
        '''
        helper method to ensure files or dirs
        are either present or removed
        '''
        for content in contents:
            log.debug('CONTENT %s', content)
            if status == 'present':
                check_job_dir = os.path.exists(content)
            elif status == 'removed':
                if os.path.exists(content):
                    check_job_dir = False
                else:
                    check_job_dir = True
            self.assertTrue(check_job_dir, msg=msg + content)

    def _add_job(self):
        '''
        helper method to add job.
        '''
        # add the job.
        opts = self.get_temp_config('master')
        opts['cachedir'] = self.TMP_CACHE_DIR
        load = {'fun_args': [], 'jid': '20160603132323715452',
                'return': True, 'retcode': 0, 'success': True,
                'cmd': '_return', 'fun': 'test.ping', 'id': 'minion'}

        add_job = salt.utils.job.store_job(opts, load)
        self.assertEqual(add_job, None)
        self._check_dir_files('Dir/file does not exist: ',
                              self.JOB_CACHE_DIR_FILES,
                              status='present')

    def test_clean_old_jobs(self):
        '''
        test to ensure jobs are removed from job cache
        '''
        self._add_job()

        # remove job
        self.assertEqual(local_cache.clean_old_jobs(), None)

        self._check_dir_files('job cache was not removed: ',
                              self.JOB_CACHE_DIR_FILES,
                              status='removed')

    def test_not_clean_new_jobs(self):
        '''
        test to ensure jobs are not removed when
        jobs dir is new
        '''
        self._add_job()

        with patch.dict(local_cache.__opts__, {'keep_jobs': 24}):
            self.assertEqual(local_cache.clean_old_jobs(), None)

            self._check_dir_files('job cache was removed: ',
                                  self.JOB_CACHE_DIR_FILES,
                                  status='present')

    def test_empty_jid_dir(self):
        '''
        test to ensure removal of empty jid dir
        '''
        # add empty jid dir
        new_jid_dir = os.path.join(self.JOBS_DIR, 'z0')
        self.EMPTY_JID_DIR.append(new_jid_dir)
        os.makedirs(new_jid_dir)

        # This needed due to a race condition in Windows
        # `os.makedirs` hasn't released the handle before
        # `local_cache.clean_old_jobs` tries to delete the new_jid_dir
        if salt.utils.platform.is_windows():
            import time
            lock_dir = new_jid_dir + '.lckchk'
            tries = 0
            while True:
                tries += 1
                if tries > 10:
                    break
                # Rename the directory and name it back
                # If it fails, the directory handle is not released, try again
                # If it succeeds, break and continue test
                try:
                    os.rename(new_jid_dir, lock_dir)
                    time.sleep(1)
                    os.rename(lock_dir, new_jid_dir)
                    break
                except WindowsError:  # pylint: disable=E0602
                    continue

        # check dir exists
        self._check_dir_files('new_jid_dir was not created',
                              self.EMPTY_JID_DIR,
                              status='present')

        # remove job
        self.assertEqual(local_cache.clean_old_jobs(), None)

        # check jid dir is removed
        self._check_dir_files('new_jid_dir was not removed',
                              self.EMPTY_JID_DIR,
                              status='removed')



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import sys
import time
import signal
import multiprocessing
import functools

# Import Salt Testing libs
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import salt libs
import salt.utils.platform
import salt.utils.process

# Import 3rd-party libs
from salt.ext import six
from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin


def die(func):
    '''
    Add proc title
    '''
    @functools.wraps(func)
    def wrapper(self):
        # Strip off the "test_" from the function name
        name = func.__name__[5:]

        def _die():
            salt.utils.process.appendproctitle('test_{0}'.format(name))
        setattr(self, 'die_' + name, _die)

    return wrapper


def incr(func):
    '''
    Increment counter
    '''
    @functools.wraps(func)
    def wrapper(self):
        # Strip off the "test_" from the function name
        name = func.__name__[5:]

        def _incr(counter, num):
            salt.utils.process.appendproctitle('test_{0}'.format(name))
            for _ in range(0, num):
                counter.value += 1
        setattr(self, 'incr_' + name, _incr)

    return wrapper


def spin(func):
    '''
    Spin indefinitely
    '''
    @functools.wraps(func)
    def wrapper(self):
        # Strip off the "test_" from the function name
        name = func.__name__[5:]

        def _spin():
            salt.utils.process.appendproctitle('test_{0}'.format(name))
            while True:
                time.sleep(1)
        setattr(self, 'spin_' + name, _spin)

    return wrapper


class TestProcessManager(TestCase):

    @spin
    def test_basic(self):
        '''
        Make sure that the process is alive 2s later
        '''
        process_manager = salt.utils.process.ProcessManager()
        process_manager.add_process(self.spin_basic)
        initial_pid = next(six.iterkeys(process_manager._process_map))
        time.sleep(2)
        process_manager.check_children()
        try:
            assert initial_pid == next(six.iterkeys(process_manager._process_map))
        finally:
            process_manager.stop_restarting()
            process_manager.kill_children()
            time.sleep(0.5)
            # Are there child processes still running?
            if process_manager._process_map.keys():
                process_manager.send_signal_to_processes(signal.SIGKILL)
                process_manager.stop_restarting()
                process_manager.kill_children()

    @spin
    def test_kill(self):
        process_manager = salt.utils.process.ProcessManager()
        process_manager.add_process(self.spin_kill)
        initial_pid = next(six.iterkeys(process_manager._process_map))
        # kill the child
        if salt.utils.platform.is_windows():
            os.kill(initial_pid, signal.SIGTERM)
        else:
            os.kill(initial_pid, signal.SIGKILL)
        # give the OS time to give the signal...
        time.sleep(0.1)
        process_manager.check_children()
        try:
            assert initial_pid != next(six.iterkeys(process_manager._process_map))
        finally:
            process_manager.stop_restarting()
            process_manager.kill_children()
            time.sleep(0.5)
            # Are there child processes still running?
            if process_manager._process_map.keys():
                process_manager.send_signal_to_processes(signal.SIGKILL)
                process_manager.stop_restarting()
                process_manager.kill_children()

    @die
    def test_restarting(self):
        '''
        Make sure that the process is alive 2s later
        '''
        process_manager = salt.utils.process.ProcessManager()
        process_manager.add_process(self.die_restarting)
        initial_pid = next(six.iterkeys(process_manager._process_map))
        time.sleep(2)
        process_manager.check_children()
        try:
            assert initial_pid != next(six.iterkeys(process_manager._process_map))
        finally:
            process_manager.stop_restarting()
            process_manager.kill_children()
            time.sleep(0.5)
            # Are there child processes still running?
            if process_manager._process_map.keys():
                process_manager.send_signal_to_processes(signal.SIGKILL)
                process_manager.stop_restarting()
                process_manager.kill_children()

    @skipIf(sys.version_info < (2, 7), 'Needs > Py 2.7 due to bug in stdlib')
    @incr
    def test_counter(self):
        counter = multiprocessing.Value('i', 0)
        process_manager = salt.utils.process.ProcessManager()
        process_manager.add_process(self.incr_counter, args=(counter, 2))
        time.sleep(1)
        process_manager.check_children()
        time.sleep(1)
        # we should have had 2 processes go at it
        try:
            assert counter.value == 4
        finally:
            process_manager.stop_restarting()
            process_manager.kill_children()
            time.sleep(0.5)
            # Are there child processes still running?
            if process_manager._process_map.keys():
                process_manager.send_signal_to_processes(signal.SIGKILL)
                process_manager.stop_restarting()
                process_manager.kill_children()


class TestThreadPool(TestCase):

    def test_basic(self):
        '''
        Make sure the threadpool can do things
        '''
        def incr_counter(counter):
            counter.value += 1
        counter = multiprocessing.Value('i', 0)

        pool = salt.utils.process.ThreadPool()
        sent = pool.fire_async(incr_counter, args=(counter,))
        self.assertTrue(sent)
        time.sleep(1)  # Sleep to let the threads do things
        self.assertEqual(counter.value, 1)
        self.assertEqual(pool._job_queue.qsize(), 0)

    def test_full_queue(self):
        '''
        Make sure that a full threadpool acts as we expect
        '''
        def incr_counter(counter):
            counter.value += 1
        counter = multiprocessing.Value('i', 0)

        # Create a pool with no workers and 1 queue size
        pool = salt.utils.process.ThreadPool(0, 1)
        # make sure we can put the one item in
        sent = pool.fire_async(incr_counter, args=(counter,))
        self.assertTrue(sent)
        # make sure we can't put more in
        sent = pool.fire_async(incr_counter, args=(counter,))
        self.assertFalse(sent)
        time.sleep(1)  # Sleep to let the threads do things
        # make sure no one updated the counter
        self.assertEqual(counter.value, 0)
        # make sure the queue is still full
        self.assertEqual(pool._job_queue.qsize(), 1)


class TestProcess(TestCase):

    @skipIf(NO_MOCK, NO_MOCK_REASON)
    def test_daemonize_if(self):
        # pylint: disable=assignment-from-none
        with patch('sys.argv', ['salt-call']):
            ret = salt.utils.process.daemonize_if({})
            self.assertEqual(None, ret)

        ret = salt.utils.process.daemonize_if({'multiprocessing': False})
        self.assertEqual(None, ret)

        with patch('sys.platform', 'win'):
            ret = salt.utils.process.daemonize_if({})
            self.assertEqual(None, ret)

        with patch('salt.utils.process.daemonize'), \
                patch('sys.platform', 'linux2'):
            salt.utils.process.daemonize_if({})
            self.assertTrue(salt.utils.process.daemonize.called)
        # pylint: enable=assignment-from-none



# -*- coding: utf-8 -*-
'''
Tests for the Mandrill execution module.
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    patch,
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.mandrill as mandrill

# Test data
TEST_SEND = {
    'result': True,
    'comment': '',
    'out': [
        {
            'status': 'sent',
            '_id': 'c4353540a3c123eca112bbdd704ab6',
            'email': 'recv@example.com',
            'reject_reason': None
        }
    ]
}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class MandrillModuleTest(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.mandrill.
    '''
    def setup_loader_modules(self):
        module_globals = {
            mandrill: {
                '__salt__': {
                    'config.merge': MagicMock(return_value={
                        'mandrill': {
                            'key': '2orgk34kgk34g'
                        }
                    })
                }
            }
        }
        if mandrill.HAS_REQUESTS is False:
            module_globals['sys.modules'] = {'requests': MagicMock()}
        return module_globals

    def test_send(self):
        '''
        Test the send function.
        '''
        mock_cmd = MagicMock(return_value=TEST_SEND)
        with patch.object(mandrill, 'send', mock_cmd) as send:
            self.assertEqual(
                send(message={
                        'subject': 'Hi',
                        'from_email': 'test@example.com',
                        'to': [
                            {'email': 'recv@example.com', 'type': 'to'}
                        ]
                    }
                ),
                TEST_SEND
            )



# -*- coding: utf-8 -*-
'''
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    patch,
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.utils.json
import salt.utils.pagerduty
import salt.modules.pagerduty as pagerduty


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PagerdutyTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.pagerduty
    '''
    def setup_loader_modules(self):
        return {pagerduty: {'__salt__': {'config.option': MagicMock(return_value=None)}}}

    def test_list_services(self):
        '''
        Test for List services belonging to this account
        '''
        with patch.object(salt.utils.pagerduty,
                          'list_items', return_value='A'):
            self.assertEqual(pagerduty.list_services(), 'A')

    def test_list_incidents(self):
        '''
        Test for List incidents belonging to this account
        '''
        with patch.object(salt.utils.pagerduty,
                          'list_items', return_value='A'):
            self.assertEqual(pagerduty.list_incidents(), 'A')

    def test_list_users(self):
        '''
        Test for List users belonging to this account
        '''
        with patch.object(salt.utils.pagerduty,
                          'list_items', return_value='A'):
            self.assertEqual(pagerduty.list_users(), 'A')

    def test_list_schedules(self):
        '''
        Test for List schedules belonging to this account
        '''
        with patch.object(salt.utils.pagerduty,
                          'list_items', return_value='A'):
            self.assertEqual(pagerduty.list_schedules(), 'A')

    def test_list_windows(self):
        '''
        Test for List maintenance windows belonging to this account
        '''
        with patch.object(salt.utils.pagerduty,
                          'list_items', return_value='A'):
            self.assertEqual(pagerduty.list_windows(), 'A')

    def test_list_policies(self):
        '''
        Test for List escalation policies belonging to this account
        '''
        with patch.object(salt.utils.pagerduty,
                          'list_items', return_value='A'):
            self.assertEqual(pagerduty.list_policies(), 'A')

    def test_create_event(self):
        '''
        Test for Create an event in PagerDuty. Designed for use in states.
        '''
        with patch.object(salt.utils.json, 'loads', return_value=['A']):
            with patch.object(salt.utils.pagerduty, 'query',
                              return_value='A'):
                self.assertListEqual(pagerduty.create_event(), ['A'])



# -*- coding: utf-8 -*-
'''
unit tests for salt.cache
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
# import integration
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    patch,
)

# Import Salt libs
import salt.payload
import salt.cache


class CacheFunctionsTest(TestCase):
    '''
    Validate the cache package functions.
    '''
    def setUp(self):
        self.opts = {'cache': 'localfs',
                     'memcache_expire_seconds': 0,
                     'memcache_max_items': 0,
                     'memcache_full_cleanup': False,
                     'memcache_debug': False}

    def test_factory_cache(self):
        ret = salt.cache.factory(self.opts)
        self.assertIsInstance(ret, salt.cache.Cache)

    def test_factory_memcache(self):
        self.opts['memcache_expire_seconds'] = 10
        ret = salt.cache.factory(self.opts)
        self.assertIsInstance(ret, salt.cache.MemCache)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class MemCacheTest(TestCase):
    '''
    Validate Cache class methods
    '''
    @patch('salt.payload.Serial')
    def setUp(self, serial_mock):  # pylint: disable=W0221
        salt.cache.MemCache.data = {}
        self.opts = {'cache': 'fake_driver',
                     'memcache_expire_seconds': 10,
                     'memcache_max_items': 3,
                     'memcache_full_cleanup': False,
                     'memcache_debug': False}
        self.cache = salt.cache.factory(self.opts)

    @patch('salt.cache.Cache.fetch', return_value='fake_data')
    @patch('salt.loader.cache', return_value={})
    def test_fetch(self, loader_mock, cache_fetch_mock):
        # Fetch value, it will be kept in cache.
        with patch('time.time', return_value=0):
            ret = self.cache.fetch('bank', 'key')
        self.assertEqual(ret, 'fake_data')
        self.assertDictEqual(salt.cache.MemCache.data, {
            'fake_driver': {
                ('bank', 'key'): [0, 'fake_data'],
                }})
        cache_fetch_mock.assert_called_once_with('bank', 'key')
        cache_fetch_mock.reset_mock()

        # Fetch again, cached value is used, time updated.
        with patch('time.time', return_value=1):
            ret = self.cache.fetch('bank', 'key')
        self.assertEqual(ret, 'fake_data')
        self.assertDictEqual(salt.cache.MemCache.data, {
            'fake_driver': {
                ('bank', 'key'): [1, 'fake_data'],
                }})
        cache_fetch_mock.assert_not_called()

        # Fetch after expire
        with patch('time.time', return_value=12):
            ret = self.cache.fetch('bank', 'key')
        self.assertEqual(ret, 'fake_data')
        self.assertDictEqual(salt.cache.MemCache.data, {
            'fake_driver': {
                ('bank', 'key'): [12, 'fake_data'],
                }})
        cache_fetch_mock.assert_called_once_with('bank', 'key')
        cache_fetch_mock.reset_mock()

    @patch('salt.cache.Cache.store')
    @patch('salt.loader.cache', return_value={})
    def test_store(self, loader_mock, cache_store_mock):
        # Fetch value, it will be kept in cache.
        with patch('time.time', return_value=0):
            self.cache.store('bank', 'key', 'fake_data')
        self.assertDictEqual(salt.cache.MemCache.data, {
            'fake_driver': {
                ('bank', 'key'): [0, 'fake_data'],
                }})
        cache_store_mock.assert_called_once_with('bank', 'key', 'fake_data')
        cache_store_mock.reset_mock()

        # Store another value.
        with patch('time.time', return_value=1):
            self.cache.store('bank', 'key2', 'fake_data2')
        self.assertDictEqual(salt.cache.MemCache.data, {
            'fake_driver': {
                ('bank', 'key'): [0, 'fake_data'],
                ('bank', 'key2'): [1, 'fake_data2'],
                }})
        cache_store_mock.assert_called_once_with('bank', 'key2', 'fake_data2')

    @patch('salt.cache.Cache.store')
    @patch('salt.cache.Cache.flush')
    @patch('salt.loader.cache', return_value={})
    def test_flush(self, loader_mock, cache_flush_mock, cache_store_mock):
        # Flush non-existing bank
        self.cache.flush('bank')
        self.assertDictEqual(salt.cache.MemCache.data, {'fake_driver': {}})
        cache_flush_mock.assert_called_once_with('bank', None)
        cache_flush_mock.reset_mock()
        # Flush non-existing key
        self.cache.flush('bank', 'key')
        self.assertDictEqual(salt.cache.MemCache.data, {'fake_driver': {}})
        cache_flush_mock.assert_called_once_with('bank', 'key')
        cache_flush_mock.reset_mock()
        # Flush existing key
        with patch('time.time', return_value=0):
            self.cache.store('bank', 'key', 'fake_data')
        self.assertEqual(salt.cache.MemCache.data['fake_driver'][('bank', 'key')],
                         [0, 'fake_data'])
        self.assertDictEqual(salt.cache.MemCache.data, {
            'fake_driver': {
                ('bank', 'key'): [0, 'fake_data'],
                }})
        self.cache.flush('bank', 'key')
        self.assertDictEqual(salt.cache.MemCache.data, {'fake_driver': {}})
        cache_flush_mock.assert_called_once_with('bank', 'key')
        cache_flush_mock.reset_mock()

    @patch('salt.cache.Cache.store')
    @patch('salt.loader.cache', return_value={})
    def test_max_items(self, loader_mock, cache_store_mock):
        # Put MAX=3 values
        with patch('time.time', return_value=0):
            self.cache.store('bank1', 'key1', 'fake_data11')
        with patch('time.time', return_value=1):
            self.cache.store('bank1', 'key2', 'fake_data12')
        with patch('time.time', return_value=2):
            self.cache.store('bank2', 'key1', 'fake_data21')
        self.assertDictEqual(salt.cache.MemCache.data['fake_driver'], {
            ('bank1', 'key1'): [0, 'fake_data11'],
            ('bank1', 'key2'): [1, 'fake_data12'],
            ('bank2', 'key1'): [2, 'fake_data21'],
            })
        # Put one more and check the oldest was removed
        with patch('time.time', return_value=3):
            self.cache.store('bank2', 'key2', 'fake_data22')
        self.assertDictEqual(salt.cache.MemCache.data['fake_driver'], {
            ('bank1', 'key2'): [1, 'fake_data12'],
            ('bank2', 'key1'): [2, 'fake_data21'],
            ('bank2', 'key2'): [3, 'fake_data22'],
            })

    @patch('salt.cache.Cache.store')
    @patch('salt.loader.cache', return_value={})
    def test_full_cleanup(self, loader_mock, cache_store_mock):
        # Enable full cleanup
        self.cache.cleanup = True
        # Put MAX=3 values
        with patch('time.time', return_value=0):
            self.cache.store('bank1', 'key1', 'fake_data11')
        with patch('time.time', return_value=1):
            self.cache.store('bank1', 'key2', 'fake_data12')
        with patch('time.time', return_value=2):
            self.cache.store('bank2', 'key1', 'fake_data21')
        self.assertDictEqual(salt.cache.MemCache.data['fake_driver'], {
            ('bank1', 'key1'): [0, 'fake_data11'],
            ('bank1', 'key2'): [1, 'fake_data12'],
            ('bank2', 'key1'): [2, 'fake_data21'],
            })
        # Put one more and check all expired was removed
        with patch('time.time', return_value=12):
            self.cache.store('bank2', 'key2', 'fake_data22')
        self.assertDictEqual(salt.cache.MemCache.data['fake_driver'], {
            ('bank2', 'key1'): [2, 'fake_data21'],
            ('bank2', 'key2'): [12, 'fake_data22'],
            })

    @patch('salt.cache.Cache.fetch', return_value='fake_data')
    @patch('salt.loader.cache', return_value={})
    def test_fetch_debug(self, loader_mock, cache_fetch_mock):
        # Recreate cache with debug enabled
        self.opts['memcache_debug'] = True
        self.cache = salt.cache.factory(self.opts)

        # Fetch 2 values (no cache hit)
        with patch('time.time', return_value=0):
            ret = self.cache.fetch('bank', 'key1')
        with patch('time.time', return_value=1):
            ret = self.cache.fetch('bank', 'key2')
        # Fetch 3 times (cache hit)
        with patch('time.time', return_value=2):
            ret = self.cache.fetch('bank', 'key2')
        with patch('time.time', return_value=3):
            ret = self.cache.fetch('bank', 'key1')
        with patch('time.time', return_value=4):
            ret = self.cache.fetch('bank', 'key1')
        # Fetch an expired value (no cache hit)
        with patch('time.time', return_value=13):
            ret = self.cache.fetch('bank', 'key2')

        # Check debug data
        self.assertEqual(self.cache.call, 6)
        self.assertEqual(self.cache.hit, 3)
